import requests

from infoworks.error import DomainError
from infoworks.sdk import url_builder
from infoworks.sdk.base_client import BaseClient
from infoworks.sdk.local_configurations import Response, ErrorCode
from infoworks.sdk.generic_response import GenericResponse
from infoworks.sdk.url_builder import get_parent_entity_url
from infoworks.sdk.utils import IWUtils


class DomainClient(BaseClient):
    def __init__(self):
        super(DomainClient, self).__init__()

    def create_domain(self, config_body):
        """
        Function to create domain
        :param config_body: JSON payload for the domain creation

        config_body_example = {
         "name": "Abhi",
         "description": "Example Domain",
         "environment_id": "d60a47e2c8438a3d6daf7958"
        }
        :return: response dict
        """
        try:
            create_domain_url = url_builder.create_domain_url(self.client_config)
            response = self.call_api("POST", create_domain_url,
                                     IWUtils.get_default_header_for_v3(self.client_config['bearer_token']),
                                     config_body)
            parsed_response = IWUtils.ejson_deserialize(
                response.content)
            if response.status_code == 200:
                return GenericResponse.parse_result(status=Response.Status.SUCCESS, response=parsed_response)
            else:
                return GenericResponse.parse_result(status=Response.Status.FAILED,
                                                    error_code=ErrorCode.GENERIC_ERROR,
                                                    error_desc=parsed_response, job_id=None)
        except Exception as e:
            raise DomainError(f"Failed to create domain" + str(e))

    def list_domains(self, params=None):
        """
        Function to list domains
        :param params: Pass the parameters like limit, filter, offset, sort_by, order_by as a dictionary
        :return: response dict
        """
        if params is None:
            params = {"limit": 20, "offset": 0}
        url_to_list_domains = url_builder.create_domain_url(
            self.client_config) + IWUtils.get_query_params_string_from_dict(params=params)
        domains_list = []
        try:
            response = IWUtils.ejson_deserialize(
                self.call_api("GET", url_to_list_domains,
                              IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])).content)
            if response is not None:
                result = response.get("result", [])
                while len(result) > 0:
                    domains_list.extend(result)
                    nextUrl = '{protocol}://{ip}:{port}{next}'.format(next=response.get('links')['next'],
                                                                      ip=self.client_config['ip'],
                                                                      port=self.client_config['port'],
                                                                      protocol=self.client_config['protocol'],
                                                                      )
                    response = IWUtils.ejson_deserialize(
                        self.call_api("GET", nextUrl, IWUtils.get_default_header_for_v3(
                            self.client_config['bearer_token'])).content)
                    result = response.get("result", [])
            return GenericResponse.parse_result(status=Response.Status.SUCCESS, response=domains_list)
        except Exception as e:
            self.logger.error("Error in listing domains")
            raise DomainError("Error in listing domains" + str(e))

    def get_domain_details(self, domain_id):
        """
        Function to get the domain details
        :param domain_id: Entity identifier for the domain
        :return: response dict
        """
        try:
            url_to_get_domain_details = url_builder.create_domain_url(self.client_config) + f"/{domain_id}"
            response = IWUtils.ejson_deserialize(self.call_api("GET", url_to_get_domain_details,
                                                               IWUtils.get_default_header_for_v3(
                                                                   self.client_config['bearer_token']),
                                                               ).content)
            result = response.get('result', False)
            if not result:
                self.logger.error(f"Failed to get the domain details for {domain_id} ")
                return GenericResponse.parse_result(status=Response.Status.FAILED, error_code=ErrorCode.USER_ERROR,
                                                    error_desc=response, job_id=None)
            else:
                return GenericResponse.parse_result(status=Response.Status.SUCCESS, response=result)
        except Exception as e:
            raise DomainError(f"Failed to get the domain details for {domain_id} " + str(e))

    def update_domain(self, domain_id, config_body):
        """
        Function to update the domain
        :param domain_id: Entity identifier for the domain
        :param config_body: JSON payload for the domain creation
        :return: response dict
        """
        try:
            url_to_update_domain = url_builder.create_domain_url(self.client_config) + f"/{domain_id}"
            response = self.call_api("PATCH", url_to_update_domain,
                                     IWUtils.get_default_header_for_v3(self.client_config['bearer_token']),
                                     config_body)
            parsed_response = IWUtils.ejson_deserialize(
                response.content)
            if response.status_code == 200:
                return GenericResponse.parse_result(status=Response.Status.SUCCESS)
            else:
                return GenericResponse.parse_result(status=Response.Status.FAILED,
                                                    error_code=ErrorCode.GENERIC_ERROR,
                                                    error_desc=parsed_response, job_id=None)
        except Exception as e:
            raise DomainError(f"Failed to update the domain {domain_id}" + str(e))

    def delete_domain(self, domain_id):
        """
        Function to delete the domain
        :param domain_id: Entity identifier for the domain
        :return: response dict
        """
        url_to_delete_domain = url_builder.create_domain_url(self.client_config) + f"/{domain_id}"
        try:
            response = self.call_api("DELETE",
                                     url_to_delete_domain,
                                     IWUtils.get_default_header_for_v3(self.client_config['bearer_token']))
            parsed_response = IWUtils.ejson_deserialize(
                response.content,
            )
            if response.status_code == 200:
                return GenericResponse.parse_result(status=Response.Status.SUCCESS)
            else:
                return GenericResponse.parse_result(status=Response.Status.FAILED,
                                                    error_code=ErrorCode.GENERIC_ERROR,
                                                    error_desc=parsed_response.get("details",
                                                                                   "Error in deleting domain")
                                                    )
        except Exception as e:
            self.logger.error(f"Error in deleting the domain {domain_id}")
            raise DomainError(f"Error in deleting the domain {domain_id} " + str(e))

    def get_sources_associated_with_domain(self, domain_id, params=None):
        """
        Function to get sources associated with the domain
        :param domain_id: Entity identifier for the domain
        :param params: Pass the parameters like limit, filter, offset, sort_by, order_by as a dictionary
        :return: response dict
        """
        if params is None:
            params = {"limit": 20, "offset": 0}
        url_to_list_sources_under_domain = url_builder.add_sources_to_domain_url(self.client_config,
                                                                                 domain_id) + IWUtils.get_query_params_string_from_dict(
            params=params)
        src_under_domain_list = []
        try:
            response = IWUtils.ejson_deserialize(
                self.call_api("GET", url_to_list_sources_under_domain,
                              IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])).content)
            if response is not None:
                result = response.get("result", [])
                while len(result) > 0:
                    src_under_domain_list.extend(result)
                    nextUrl = '{protocol}://{ip}:{port}{next}'.format(next=response.get('links')['next'],
                                                                      ip=self.client_config['ip'],
                                                                      port=self.client_config['port'],
                                                                      protocol=self.client_config['protocol'],
                                                                      )
                    response = IWUtils.ejson_deserialize(
                        self.call_api("GET", nextUrl, IWUtils.get_default_header_for_v3(
                            self.client_config['bearer_token'])).content)
                    result = response.get("result", [])
            return GenericResponse.parse_result(status=Response.Status.SUCCESS, response=src_under_domain_list)
        except Exception as e:
            self.logger.error("Error in listing sources under domain")
            raise DomainError("Error in listing sources under domain" + str(e))

    def add_source_to_domain(self, domain_id, config_body):
        """
        Function to add source to the domain
        :param domain_id: Entity identifier for the domain
        :param config_body: JSON payload
        config_body_example =
            {
             "entity_ids": ["33164bbaa811058db5c3941d"]
            }
        :return: response dict
        """
        try:
            url_to_add_source_domain = url_builder.add_sources_to_domain_url(self.client_config, domain_id)
            response = self.call_api("POST", url_to_add_source_domain,
                                     IWUtils.get_default_header_for_v3(self.client_config['bearer_token']),
                                     config_body)
            parsed_response = IWUtils.ejson_deserialize(
                response.content)
            if response.status_code == 200:
                return GenericResponse.parse_result(status=Response.Status.SUCCESS)
            else:
                return GenericResponse.parse_result(status=Response.Status.FAILED,
                                                    error_code=ErrorCode.GENERIC_ERROR,
                                                    error_desc=parsed_response)
        except Exception as e:
            raise DomainError(f"Failed to add source to the domain {domain_id}" + str(e))

    def remove_source_from_domain(self, domain_id, config_body):
        """
        Function to remove source from the domain
        :param domain_id: Entity identifier for the domain
        :param config_body: JSON payload

        config_body_example =
            {
             "entity_ids": ["33164bbaa811058db5c3941d"]
            }
        :return: response dict
        """
        try:
            url_to_delete_source_domain = url_builder.add_sources_to_domain_url(self.client_config, domain_id)
            response = self.call_api("DELETE", url_to_delete_source_domain,
                                     IWUtils.get_default_header_for_v3(self.client_config['bearer_token']),
                                     config_body)
            parsed_response = IWUtils.ejson_deserialize(
                response.content)
            if response.status_code == 200:
                return GenericResponse.parse_result(status=Response.Status.SUCCESS)
            else:
                return GenericResponse.parse_result(status=Response.Status.FAILED,
                                                    error_code=ErrorCode.GENERIC_ERROR,
                                                    error_desc=parsed_response)
        except Exception as e:
            raise DomainError(f"Failed to remove source from the domain {domain_id}" + str(e))

    def get_data_connection(self, domain_id, data_connection_id=None, params=None):
        """
        Function to create data connection in the domain
        :param domain_id: Entity identifier for the domain
        :param data_connection_id: id of dataconnection for which config has to be fetched
        :param params: Pass the parameters like limit, filter, offset, sort_by, order_by as a dictionary
        :type: JSON dict
        :return: response dict
        """
        if params is None:
            params = {"limit": 20, "offset": 0}

        if data_connection_id is None:
            url_to_get_data_connection = url_builder.create_data_connection(
                self.client_config, domain_id) + IWUtils.get_query_params_string_from_dict(params=params)
        else:
            url_to_get_data_connection = url_builder.create_data_connection(
                self.client_config, domain_id) + f"/{data_connection_id}"
        dataconnection_list = []
        try:
            response = IWUtils.ejson_deserialize(self.call_api("GET", url_to_get_data_connection,
                                     IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])).content)
            if response is not None:
                result = response.get("result", [])
                if data_connection_id is not None:
                    dataconnection_list.extend([result])
                else:
                    while len(result) > 0:
                        dataconnection_list.extend(result)
                        nextUrl = '{protocol}://{ip}:{port}{next}'.format(next=response.get('links')['next'],
                                                                          ip=self.client_config['ip'],
                                                                          port=self.client_config['port'],
                                                                          protocol=self.client_config['protocol'],
                                                                          )
                        response = IWUtils.ejson_deserialize(
                            self.call_api("GET", nextUrl, IWUtils.get_default_header_for_v3(
                                self.client_config['bearer_token'])).content)
                        result = response.get("result", [])
            return GenericResponse.parse_result(status=Response.Status.SUCCESS, response=dataconnection_list)
        except Exception as e:
            raise DomainError(f"Failed to get data connection details" + str(e))

    def create_data_connection(self, domain_id, config_body):
        """
        Function to create data connection in the domain
        :param domain_id: Entity identifier for the domain
        :param config_body: JSON payload
        :return: response dict
        """
        try:
            create_data_connection_url = url_builder.create_data_connection(self.client_config, domain_id)
            response = self.call_api("POST", create_data_connection_url,
                                     IWUtils.get_default_header_for_v3(self.client_config['bearer_token']),
                                     config_body)
            parsed_response = IWUtils.ejson_deserialize(
                response.content)
            if response.status_code == 200:
                return GenericResponse.parse_result(status=Response.Status.SUCCESS, response=parsed_response)
            else:
                return GenericResponse.parse_result(status=Response.Status.FAILED,
                                                    error_code=ErrorCode.GENERIC_ERROR,
                                                    error_desc=parsed_response, job_id=None)
        except Exception as e:
            raise DomainError(f"Failed to create data connection" + str(e))

    def delete_data_connection(self, domain_id, data_connection_id):
        """
        Function to delete data connection in the domain
        :param domain_id: Entity identifier for the domain
        :param data_connection_id: Entity identifier for the data connection
        :return: response dict
        """
        try:
            create_data_connection_url = url_builder.create_data_connection(self.client_config, domain_id).strip(
                "/") + f"/{data_connection_id}"
            response = self.call_api("DELETE", create_data_connection_url,
                                     IWUtils.get_default_header_for_v3(self.client_config['bearer_token']),
                                     )
            parsed_response = IWUtils.ejson_deserialize(
                response.content)
            if response.status_code == 200:
                return GenericResponse.parse_result(status=Response.Status.SUCCESS)
            else:
                return GenericResponse.parse_result(status=Response.Status.FAILED,
                                                    error_code=ErrorCode.GENERIC_ERROR,
                                                    error_desc=parsed_response, job_id=None)
        except Exception as e:
            raise DomainError(f"Failed in deleting data connection" + str(e))

    def get_parent_entity_id(self, json_obj):
        """
        Function to get domain
        :param json_obj: Example: {"entity_id": workflow_id, "entity_type": "workflow"}
        :return: response dict
        """
        parent_entity_url = get_parent_entity_url(self.client_config)
        response = requests.request("GET", parent_entity_url, data=json_obj)
        if response.status_code == 200 and len(response.json().get("result", [])) > 0:
            result = response.json().get("result", [])
            if len(result) > 0:
                domain_id = result[0]["entity_id"]
                self.logger.info("Domain ID is {} ".format(domain_id))
                return domain_id
            else:
                self.logger.info("Domain ID is {} ".format(None))
                return None

    def get_domain_id(self, domain_name):
        """
        Function to get domain id using domain name
        :param domain_name: Name of the domain
        :return: Entity identifier of the domain
        """
        params = {"filter": {"name": domain_name}}
        url_to_list_domains = url_builder.create_domain_url(
            self.client_config) + IWUtils.get_query_params_string_from_dict(params=params)
        try:
            response = IWUtils.ejson_deserialize(
                self.call_api("GET", url_to_list_domains,
                              IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])).content)
            if response is not None:
                result = response.get("result", [])
                domain_id = result[0]["id"]
                return domain_id
        except Exception as e:
            self.logger.error("Error in finding domain id")
            raise DomainError("Error in finding domain id" + str(e))



import copy
import os
from infoworks.sdk.utils import IWUtils
from infoworks.sdk.url_builder import get_parent_entity_url, list_domains_url, configure_pipeline_url, \
    configure_workflow_url, \
    configure_source_url, get_environment_details, get_environment_storage_details, get_environment_compute_details, \
    get_environment_interactive_compute_details, get_source_configurations_url, get_pipeline_url, \
    get_data_connection, source_info, list_users_url
from infoworks.sdk.cicd.cicd_response import CICDResponse
import json


class Utils:
    def __init__(self, serviceaccountemail):
        self.serviceaccountemail = serviceaccountemail

    def get_domain_id(self, cicd_client, json_obj):
        parent_entity_url = get_parent_entity_url(cicd_client.client_config)
        response = cicd_client.call_api("GET", parent_entity_url,
                                        IWUtils.get_default_header_for_v3(cicd_client.client_config['bearer_token']),
                                        data=json_obj)
        parsed_response = IWUtils.ejson_deserialize(response.content)
        if response.status_code == 200 and len(parsed_response.get("result", [])) > 0:
            result = parsed_response.get("result", [])
            if len(result) > 0:
                domain_id = result[0]["entity_id"]
                cicd_client.logger.info("Domain ID is {} ".format(domain_id))
                return domain_id
            else:
                cicd_client.logger.info("Domain ID is {} ".format(None))
                return None

    def get_env_entities_names(self, cicd_client, environment_id, environment_compute_template_id,
                               environment_storage_id):
        env_name, storage_name, compute_name = None, None, None
        if environment_id:
            response = cicd_client.call_api("GET", get_environment_details(cicd_client.client_config, environment_id),
                                            IWUtils.get_default_header_for_v3(
                                                cicd_client.client_config['bearer_token']))
            parsed_response = IWUtils.ejson_deserialize(response.content)
            if response.status_code == 200:
                result = parsed_response.get("result", None)
                if result:
                    env_name = result["name"]
        if environment_storage_id:
            response = cicd_client.call_api("GET",
                                            get_environment_storage_details(cicd_client.client_config, environment_id,
                                                                            environment_storage_id),
                                            IWUtils.get_default_header_for_v3(
                                                cicd_client.client_config['bearer_token']))
            parsed_response = IWUtils.ejson_deserialize(response.content)
            if response.status_code == 200:
                result = parsed_response.get("result", None)
                if result:
                    storage_name = result["name"]
        if environment_compute_template_id:
            response = cicd_client.call_api("GET",
                                            get_environment_compute_details(cicd_client.client_config, environment_id,
                                                                            environment_compute_template_id),
                                            IWUtils.get_default_header_for_v3(
                                                cicd_client.client_config['bearer_token']))
            parsed_response = IWUtils.ejson_deserialize(response.content)
            if response.status_code == 200:
                result = parsed_response.get("result", None)
                if result:
                    compute_name = result["name"]
                else:
                    response = cicd_client.call_api("GET",
                                                    get_environment_interactive_compute_details(
                                                        cicd_client.client_config,
                                                        environment_id,
                                                        environment_compute_template_id),
                                                    IWUtils.get_default_header_for_v3(
                                                        cicd_client.client_config['bearer_token']))
                    parsed_response = IWUtils.ejson_deserialize(response.content)
                    if response.status_code == 200:
                        result = parsed_response.get("result", None)
                        if result:
                            compute_name = result["name"]

        return env_name, storage_name, compute_name

    def get_env_details(self, cicd_client, entity_id, entity_type, domain_id=None):
        environment_id, compute_template_id, storage_id = None, None, None
        if entity_type == "pipeline":
            get_pipeline_details_url = get_pipeline_url(cicd_client.client_config, domain_id, entity_id)
            response = cicd_client.call_api("GET", get_pipeline_details_url,
                                            IWUtils.get_default_header_for_v3(
                                                cicd_client.client_config['bearer_token']))
            parsed_response = IWUtils.ejson_deserialize(response.content)
            if response.status_code == 200 and len(parsed_response.get("result", [])) > 0:
                result = parsed_response.get("result", [])
                if len(result) > 0:
                    environment_id = result.get("environment_id",None)
                    storage_id = result.get("storage_id",None)
                    compute_template_id = result.get("compute_template_id",None)
                    cicd_client.logger.info(
                        "Environment ID is {} Storage ID is {} Compute Template Id is {}".format(environment_id,
                                                                                                 storage_id,
                                                                                                 compute_template_id))
        return environment_id, compute_template_id, storage_id

    def get_dataconnection_properties(self, sub_type, properties_obj):
        final_obj = {}
        if sub_type.lower() == "snowflake":
            for key in properties_obj.keys():
                if key == "user_name":
                    final_obj["username"] = properties_obj["user_name"]
                else:
                    final_obj[key] = properties_obj[key]
        elif sub_type.lower() == "postgres":
            for key in properties_obj.keys():
                if key == "user_name":
                    final_obj["username"] = properties_obj["user_name"]
                else:
                    final_obj[key] = properties_obj[key]
        elif sub_type.lower() == "bigquery":
            for key in properties_obj.keys():
                if key == "uploadOption":
                    final_obj["upload_option"] = properties_obj["uploadOption"]
                elif key == "serverPath":
                    final_obj["server_path"] = properties_obj["serverPath"]
                else:
                    final_obj[key] = properties_obj[key]
            if "file_details" in final_obj:
                del final_obj["file_details"]

        return final_obj

    def dump_to_file(self, cicd_client, entity_type, domain_id, entity_id, replace_words, target_file_path):
        response_to_return = {}
        filename = None
        environment_id, environment_compute_template_id, environment_storage_id = None, None, None
        if entity_type == "pipeline":
            url_to_config = configure_pipeline_url(cicd_client.client_config, domain_id, entity_id)
        elif entity_type == "workflow":
            url_to_config = configure_workflow_url(cicd_client.client_config, domain_id, entity_id)
        elif entity_type == "source":
            url_to_config = configure_source_url(cicd_client.client_config, entity_id)
        else:
            return None
        cicd_client.logger.info(
            "URL to get configurations of entity {} of type {} is {}".format(entity_id, entity_type, url_to_config))
        response = cicd_client.call_api("GET", url_to_config, IWUtils.get_default_header_for_v3(
            cicd_client.client_config[
                'bearer_token']))

        parsed_response = IWUtils.ejson_deserialize(response.content)
        if response.status_code == 200:
            status = "SUCCESS"
        else:
            status = "FAILED"
        response_to_return["get_configuration_entity_response"] = CICDResponse.parse_result(status=status,
                                                                                            entity_id=entity_id,
                                                                                            response=parsed_response)
        configuration_obj = parsed_response.get('result', {})
        # print(configuration_obj)
        if len(configuration_obj) > 0:
            if entity_type == "source":
                get_src_details_url = source_info(cicd_client.client_config, entity_id)
                response = cicd_client.call_api("GET", get_src_details_url,
                                                IWUtils.get_default_header_for_v3(
                                                    cicd_client.client_config['bearer_token']))
                parsed_response = IWUtils.ejson_deserialize(response.content)
                if response.status_code == 200:
                    status = "SUCCESS"
                else:
                    status = "FAILED"
                response_to_return["get_source_details_response"] = CICDResponse.parse_result(status=status,
                                                                                              entity_id=entity_id,
                                                                                              response=parsed_response)
                if response.status_code == 200 and len(parsed_response.get("result", [])) > 0:
                    result = parsed_response.get("result", [])
                    if len(result) > 0:
                        data_lake_path = result["data_lake_path"]
                        configuration_obj["configuration"]["source_configs"]["data_lake_path"] = data_lake_path
                        configuration_obj["filter_tables_properties"] = result.get("filter_tables_properties", {})

                # Check if any table has export configurations. Works for postgres/snowflake/synapse
                # Did not test for cosmos,delimited
                for table_config in configuration_obj["configuration"]["table_configs"]:
                    if len(table_config["configuration"].get("export_configuration", [])) > 0:
                        if table_config["configuration"]["export_configuration"].get("target_type", "") in ["SNOWFLAKE",
                                                                                                            "POSTGRES",
                                                                                                            "AZURE_SQL_DW"]:
                            table_config["configuration"]["export_configuration"]["connection"]["password"] = None
                        else:
                            pass  # TO_DO
                steps_to_run = {"configure_csv_source": True, "import_source_configuration": True,
                                "configure_rdbms_source_connection": True,
                                "test_source_connection": True, "browse_source_tables": True,
                                "add_tables_to_source": True,
                                "configure_tables_and_tablegroups": True}
                configuration_obj["steps_to_run"] = steps_to_run
                filename = "{}_".format(entity_type) + configuration_obj["configuration"]["entity"][
                    "entity_name"] + ".json"
                target_file_path = os.path.join(target_file_path, entity_type, filename)
                response = cicd_client.call_api("GET",
                                                get_source_configurations_url(cicd_client.client_config, entity_id),
                                                IWUtils.get_default_header_for_v3(
                                                    cicd_client.client_config['bearer_token']))
                parsed_response = IWUtils.ejson_deserialize(response.content)
                if response.status_code == 200:
                    status = "SUCCESS"
                    result = parsed_response.get("result", None)
                    if result:
                        environment_id = result["environment_id"]
                        environment_storage_id = result["storage_id"]
                else:
                    status = "FAILED"
                response_to_return["get_source_configurations_response"] = CICDResponse.parse_result(status=status,
                                                                                                     entity_id=entity_id,
                                                                                                     response=parsed_response)
            else:
                if entity_type == "pipeline":
                    environment_id, environment_compute_template_id, environment_storage_id = self.get_env_details(
                        cicd_client, entity_id,
                        entity_type,
                        domain_id)
                    if any([environment_id, environment_compute_template_id, environment_storage_id]):
                        status = "SUCCESS"
                    else:
                        status = "FAILED"
                    response = f"Found environment details: environment_id {environment_id}, environment_compute_template_id {environment_compute_template_id}, environment_storage_id {environment_storage_id}"
                    response_to_return["get_env_details_response"] = CICDResponse.parse_result(status=status,
                                                                                               entity_id=entity_id,
                                                                                               response=response)
                    # Check if there are any data connections
                    list_of_dataconnections = [item for item in configuration_obj["configuration"]["iw_mappings"] if
                                               item["entity_type"].lower() == "data_connection"]
                    if len(list_of_dataconnections) > 0:
                        configuration_obj["dataconnection_configurations"] = []
                        for dataconn in list_of_dataconnections:
                            dataconnection_obj = {}
                            entity_id = dataconn["entity_id"]
                            get_data_connection_url = get_data_connection(cicd_client.client_config, domain_id,
                                                                          entity_id)
                            response = cicd_client.call_api("GET", get_data_connection_url,
                                                            IWUtils.get_default_header_for_v3(
                                                                cicd_client.client_config['bearer_token']))
                            parsed_response = IWUtils.ejson_deserialize(response.content)
                            if response.status_code == 200:
                                status = "SUCCESS"
                            else:
                                status = "FAILED"
                            response_to_return["get_data_connection_details_response"] = CICDResponse.parse_result(
                                status=status,
                                entity_id=entity_id,
                                response=parsed_response)
                            if response.status_code == 200 and len(parsed_response.get("result", [])) > 0:
                                result = parsed_response.get("result", [])
                                if len(result) > 0:
                                    result["properties"] = self.get_dataconnection_properties(result["sub_type"],
                                                                                              result["properties"])
                                for key in ['name', 'type', 'sub_type', 'properties']:
                                    dataconnection_obj[key] = result[key]
                                configuration_obj["dataconnection_configurations"].append(
                                    copy.deepcopy(dataconnection_obj))
                domains_url_base = list_domains_url(cicd_client.client_config)
                filter_condition = IWUtils.ejson_serialize({"_id": domain_id})
                domains_url = domains_url_base + f"?filter={{filter_condition}}".format(
                    filter_condition=filter_condition)
                response = cicd_client.call_api("GET", domains_url,
                                                IWUtils.get_default_header_for_v3(
                                                    cicd_client.client_config['bearer_token']))
                parsed_response = IWUtils.ejson_deserialize(response.content)
                if response.status_code == 200:
                    status = "SUCCESS"
                else:
                    status = "FAILED"
                response_to_return["get_domain_details_response"] = CICDResponse.parse_result(status=status,
                                                                                              entity_id=entity_id,
                                                                                              response=parsed_response)
                existing_domain_name = None
                if response.status_code == 200 and len(parsed_response.get("result", [])) > 0:
                    result = parsed_response.get("result", [])
                    existing_domain_name = result[0]["name"]
                    if entity_type == "workflow":
                        environment_ids = result[0]["environment_ids"]
                if existing_domain_name:
                    filename = existing_domain_name + "#{}_".format(entity_type) + \
                               configuration_obj["configuration"]["entity"][
                                   "entity_name"] + ".json"
                    target_file_path = os.path.join(target_file_path, entity_type, filename)

            if entity_type == "workflow":
                storage_name, compute_name = None, None
                environment_names = []
                if len(environment_ids) > 0:
                    for environment_id in environment_ids:
                        env_name, storage_name, compute_name = self.get_env_entities_names(cicd_client, environment_id,
                                                                                           None, None)
                        environment_names.append(env_name)
                        print(env_name, storage_name, compute_name)
                    configuration_obj["environment_configurations"] = {"environment_name": environment_names,
                                                                       "environment_compute_template_name": compute_name,
                                                                       "environment_storage_name": storage_name}
                else:
                    print("in else")
            else:
                env_name, storage_name, compute_name = self.get_env_entities_names(
                    cicd_client, environment_id,
                    environment_compute_template_id,
                    environment_storage_id)

                configuration_obj["environment_configurations"] = {"environment_name": env_name,
                                                                   "environment_compute_template_name": compute_name,
                                                                   "environment_storage_name": storage_name}

            filter_condition = IWUtils.ejson_serialize(
                {"$or": [{"_id": configuration_obj["configuration"]["export"]["exported_by"]},
                         {"profile.name": configuration_obj["configuration"]["export"]["exported_by"]}]})
            url_to_list_users_base = list_users_url(cicd_client.client_config)
            url_to_list_users = url_to_list_users_base + f"?filter={{filter_condition}}".format(
                filter_condition=filter_condition)
            response = cicd_client.call_api("GET", url_to_list_users,
                                            IWUtils.get_default_header_for_v3(
                                                cicd_client.client_config['bearer_token']))
            parsed_response = IWUtils.ejson_deserialize(response.content)
            configuration_obj["user_email"] = self.serviceaccountemail
            if response.status_code == 200 and len(parsed_response.get("result", [])) > 0:
                result = parsed_response.get("result", [])
                configuration_obj["user_email"] = result[0]["profile"].get("email", "admin@infoworks.io")
            try:
                if filename is not None and target_file_path is not None:
                    cicd_client.logger.info("{} {}".format(filename, target_file_path))
                    print(f"Dumping file {target_file_path}")
                    with open(target_file_path, 'w') as file_ptr:
                        contents_to_write = IWUtils.ejson_serialize(configuration_obj)
                        if replace_words != "":
                            for key, value in [item.split("->") for item in replace_words.split(";")]:
                                contents_to_write = contents_to_write.replace(key, value)
                        file_ptr.write(contents_to_write)
                    cicd_client.logger.info("Configurations have been dumped")
                    print("Configurations have been dumped")
            except Exception as e:
                cicd_client.logger.error(str(e))
                print(str(e))
        else:
            cicd_client.logger.info("Unable to dump the configurations")
            print("Unable to dump the configurations")
        #for item in response_to_return:
        #    print(item, json.dumps(response_to_return[item]))
        return filename, configuration_obj



class SourceResponse:
    def __init__(self):
        pass

    @staticmethod
    def parse_result(status=None, error_code=None, error_desc=None, job_id=None, source_id=None, response=None):
        result = {
            'error': {
                'error_code': error_code,
                'error_desc': error_desc
            },
            'result': {
                'job_id': job_id,
                'status': status,
                'source_id': str(source_id) if source_id is not None else "",
                'response': response
            }
        }
        return result



import logging
import urllib.request, urllib.parse, urllib.error
import json

import requests
from bson import ObjectId
from infoworks.sdk import ejson, local_configurations
from infoworks.sdk.local_configurations import Response
from infoworks.sdk import url_builder


def validate_client_configs(client_config, bearer_token=None):
    """
    validates client configurations against null values
    :param client_config: client configurations
    :type client_config: dict
    :param bearer_token: bearer_token to authenticate the requests
    :return: boolean true of false based on success
    """
    if bearer_token is not None:
        client_config['auth_token'] = bearer_token

    for element in client_config.items():
        if not element[1]:
            return False, 'Missing {parameter} in client configurations'.format(parameter=element)
    return True, 'YAY!!!! client configurations are correct'


class IWUtils(object):

    def __init__(self):
        pass

    @staticmethod
    # returns a encoded string associated with the value provided. Throws exception in case of error.
    def encode(value):
        return urllib.parse.quote(value)

    @staticmethod
    def ejson_deserialize(data):
        try:
            cust_hooks = {
                'oid': ObjectId
            }
            return ejson.loads(data, custom_type_hooks=cust_hooks)
        except Exception as e:
            print('Exception: ' + str(e))
            raise RuntimeError(str(e))

    @staticmethod
    def ejson_serialize(data):
        try:
            cust_hooks = [(ObjectId, 'oid', str)]
            return ejson.dumps(data, custom_type_hooks=cust_hooks)
        except Exception as e:
            print('Exception: ' + str(e))
            raise RuntimeError(str(e))

    @staticmethod
    # returns polling frequency for polling methods
    def get_polling_frequency_in_secs():
        return local_configurations.POLLING_FREQUENCY_IN_SEC

    @staticmethod
    def get_polling_failure_retries():
        return local_configurations.NUM_POLLING_RETRIES

    @staticmethod
    def get_default_header_for_v3(delegation_token):
        return {'Authorization': 'Bearer {del_token}'.format(del_token=delegation_token)}

    @staticmethod
    def get_query_params_string_from_dict(params=None):
        if not params:
            return ""
        string = "?"
        string = string + "&limit=" + str(params.get('limit')) if params.get('limit') else string
        string = string + "&sort_by=" + str(params.get('sort_by')) if params.get('sort_by') else string
        string = string + "&order_by=" + str(params.get('order_by')) if params.get('order_by') else string
        string = string + "&offset=" + str(params.get('offset')) if params.get('offset') else string
        string = string + "&filter=" + urllib.parse.quote(json.dumps(params.get('filter'))) if params.get(
            'filter') else string
        string = string + "&projections=" + urllib.parse.quote(json.dumps(params.get('projections'))) if params.get(
            'projections') else string
        return string

    @staticmethod
    def get_parent_entity(entity_type, entity_id, client_config):
        try:
            if entity_id is None or entity_type is None:
                return Response.Status.ERROR, 'Failed to get Parent Id for: {entity_type}: {entity_id}'.format(
                    entity_type=entity_type, entity_id=entity_id)
            data = {
                'entity_type': entity_type,
                'entity_id': entity_id
            }
            result = requests.get(
                url_builder.get_parent_entity_url(client_config),
                json=data
            )

            response = IWUtils.ejson_deserialize(result.content)
            if str(result.status_code) != '200' or 'result' not in response:
                logging.error('Failed to get Parent Id for: {}: {}', entity_type, entity_id)
                return Response.Status.ERROR, 'Failed to get Parent Id for: {entity_type}: {entity_id}'.format(
                    entity_type=entity_type, entity_id=entity_id)
            return Response.Status.SUCCESS, response.get('result', [])
        except Exception as e:
            logging.error('Exception:' + str(e))
            return Response.Status.ERROR, 'Exception: ' + str(e)

    @staticmethod
    def is_json(myjson):
        try:
            json.loads(myjson)
        except ValueError as e:
            return False
        return True



import traceback

from infoworks.sdk import url_builder
from infoworks.sdk.base_client import BaseClient
from infoworks.sdk.cicd.upload_configurations.cdata_source import CdataSource
from infoworks.sdk.cicd.upload_configurations.csv_source import CSVSource
from infoworks.sdk.cicd.upload_configurations.rdbms_source import RDBMSSource
from infoworks.sdk.cicd.upload_configurations.salesforce_source import SalesforceSource
from infoworks.sdk.generic_response import GenericResponse
from infoworks.sdk.utils import IWUtils
from infoworks.sdk.local_configurations import Response
import os.path
import queue
import threading


class WrapperSource(BaseClient):
    def __init__(self):
        super().__init__()

    def __wrapper_get_environment_details(self, environment_id=None, params=None):
        if params is None and environment_id is None:
            params = {"limit": 20, "offset": 0}
        url_to_list_environments = url_builder.get_environment_details(
            self.client_config, environment_id) + IWUtils.get_query_params_string_from_dict(params=params)
        env_details = []
        try:
            response = IWUtils.ejson_deserialize(
                self.call_api("GET", url_to_list_environments,
                              IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])).content)
            if response is not None:
                result = response.get("result", [])
                if environment_id is not None:
                    env_details.extend(result)
                else:
                    while len(result) > 0:
                        env_details.extend(result)
                        nextUrl = '{protocol}://{ip}:{port}{next}'.format(next=response.get('links')['next'],
                                                                          ip=self.client_config['ip'],
                                                                          port=self.client_config['port'],
                                                                          protocol=self.client_config['protocol'],
                                                                          )
                        response = IWUtils.ejson_deserialize(
                            self.call_api("GET", nextUrl, IWUtils.get_default_header_for_v3(
                                self.client_config['bearer_token'])).content)
                        result = response.get("result", [])
            return GenericResponse.parse_result(status=Response.Status.SUCCESS, response=env_details)
        except Exception as e:
            self.logger.error("Error in getting environment details")
            print("Error in getting environment details")

    def __wrapper_get_storage_details(self, environment_id, storage_id=None, params=None):
        if params is None and storage_id is None:
            params = {"limit": 20, "offset": 0}
        url_to_list_storages = url_builder.get_environment_storage_details(
            self.client_config, environment_id, storage_id) + IWUtils.get_query_params_string_from_dict(params=params)
        storage_details = []
        try:
            response = IWUtils.ejson_deserialize(
                self.call_api("GET", url_to_list_storages,
                              IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])).content)
            if response is not None:
                result = response.get("result", [])
                if storage_id is not None:
                    storage_details.extend(result)
                else:
                    while len(result) > 0:
                        storage_details.extend(result)
                        nextUrl = '{protocol}://{ip}:{port}{next}'.format(next=response.get('links')['next'],
                                                                          ip=self.client_config['ip'],
                                                                          port=self.client_config['port'],
                                                                          protocol=self.client_config['protocol'],
                                                                          )
                        response = IWUtils.ejson_deserialize(
                            self.call_api("GET", nextUrl, IWUtils.get_default_header_for_v3(
                                self.client_config['bearer_token'])).content)
                        result = response.get("result", [])
            return GenericResponse.parse_result(status=Response.Status.SUCCESS, response=storage_details)
        except Exception as e:
            self.logger.error("Error in getting storage details")
            print("Error in getting storage details")

    def cicd_upload_source_configurations(self, configuration_file_path, override_configuration_file=None,
                                          export_lookup=False, replace_words="", read_passwords_from_secrets=False,
                                          env_tag="", secret_type="",config_ini_path=None):
        """
        Function to create and configure source using the source configuration JSON file
        :param configuration_file_path: Path of the file with source configurations to be imported
        :param override_configuration_file: Path of the file with override keys
        :param export_lookup: True/False. True if for each table in the source export, override keys have to be looked up from override_configuration_file and passwords from secrets
        :param replace_words: Pass the strings to be replaced in the configuration file. Example: DEV->PROD;dev->prod
        :param read_passwords_from_secrets: True/False. If True all the source related passwords are read from encrypted file name passed
        """
        response_to_return ={}
        try:
            env_id = self.client_config.get("default_environment_id", None)
            storage_id = self.client_config.get("default_storage_id", None)
            with open(configuration_file_path, 'r') as file:
                json_string = file.read()
            configuration_obj = IWUtils.ejson_deserialize(json_string)
            environment_configurations = configuration_obj["environment_configurations"]
            if env_id is None and "environment_mappings" in self.mappings:
                env_name = self.mappings["environment_mappings"].get(environment_configurations["environment_name"],
                                                                     environment_configurations["environment_name"])
                if env_name is not None:
                    result = self.__wrapper_get_environment_details(params={"filter": {"name": env_name}})
                    env_id = result["result"]["response"][0]["id"] if len(result["result"]["response"]) > 0 else None
                    response_to_return["get_environment_entity_response"] = result
            if storage_id is None and "storage_mappings" in self.mappings:
                storage_name = self.mappings["storage_mappings"].get(
                    environment_configurations["environment_storage_name"],
                    environment_configurations["environment_storage_name"])
                if storage_name is not None:
                    result = self.__wrapper_get_storage_details(environment_id=env_id,
                                                                params={"filter": {"name": storage_name}})
                    storage_id = result["result"]["response"][0]["id"] if len(result["result"]["response"]) > 0 else None
                    response_to_return["get_storage_entity_response"] = result
            if env_id is None or storage_id is None:
                print("No env id or storage id found")
                raise Exception("No env id or storage id found")

            source_type = configuration_obj["configuration"]["source_configs"]["type"]
            source_sub_type = configuration_obj["configuration"]["source_configs"]["sub_type"]
            if source_type == "file" and source_sub_type == "structured":
                source_obj = CSVSource(env_id, storage_id, configuration_file_path, self.secrets_config, replace_words)
                create_source_response = source_obj.create_csv_source(self)
                source_id=create_source_response["result"]["source_id"]
                if source_id is not None:
                    # Proceed to configure the source connection details
                    source_connection_configurations_response = source_obj.configure_csv_source(self, source_id, self.mappings,
                                                    read_passwords_from_secrets=read_passwords_from_secrets,
                                                    env_tag=env_tag, secret_type=secret_type,config_ini_path=config_ini_path)
                    if source_connection_configurations_response["result"]["status"].upper() == "SUCCESS":
                        print("Successfully configured the connection details")
                        self.logger.info("Successfully configured the connection details")
                        # Proceed to configure tables and table groups
                        source_import_configuration_response = source_obj.import_source_configuration(self, source_id, self.mappings,
                                                                        override_configuration_file, export_lookup,
                                                                        read_passwords_from_secrets)
                        if source_import_configuration_response["result"]["status"].upper() == "SUCCESS":
                            self.logger.info("Configured source successfully")
                            print("Configured Source successfully!")
                        else:
                            self.logger.info("Failed to configure source")
                            print("Failed to configure source")
                            print(source_import_configuration_response)
                        response_to_return[
                            "source_import_configuration_response"] = source_import_configuration_response
                    else:
                        print("Failed to configure the source connection details")
                        print(source_connection_configurations_response)
                    response_to_return["source_connection_configurations_response"] = source_connection_configurations_response

                else:
                    print("Failed to create source")
                response_to_return["create_source_response"] = create_source_response
            elif source_type == "rdbms" and source_sub_type!="snowflake":
                source_obj = RDBMSSource()
                source_obj.set_variables(env_id, storage_id, configuration_file_path, self.secrets_config,
                                         replace_words)
                source_creation_response = source_obj.create_rdbms_source(self)
                source_id = source_creation_response["result"]["source_id"]
                if source_id is not None:
                    # Proceed to configure the source connection details
                    source_connection_configurations_response = source_obj.configure_rdbms_source_connection(self, source_id, override_configuration_file,
                                                                    read_passwords_from_secrets=read_passwords_from_secrets,
                                                                    env_tag=env_tag,
                                                                    secret_type=secret_type,config_ini_path=config_ini_path)
                    if source_connection_configurations_response["result"]["status"].upper() == "SUCCESS":
                        print("Successfully configured the connection details")
                        self.logger.info("Successfully configured the connection details")
                        source_test_connection_response = source_obj.test_source_connection(self, source_id)
                        if source_test_connection_response["result"]["status"].upper() == "SUCCESS":
                            source_browse_source_tables_response = source_obj.browse_source_tables(self, source_id)
                            if source_browse_source_tables_response["result"]["status"].upper() == "SUCCESS":
                                status = source_obj.add_tables_to_source(self, source_id)
                                if status == "SUCCESS":
                                    self.logger.info("Added tables to source successfully")
                                    print("Added tables to source successfully")
                                status = source_obj.configure_tables_and_tablegroups(self, source_id,
                                                                                     override_configuration_file,
                                                                                     export_lookup, self.mappings,
                                                                                     read_passwords_from_secrets,
                                                                                     env_tag=env_tag,
                                                                                     secret_type=secret_type)
                                if status == "SUCCESS":
                                    self.logger.info("Configured source successfully")
                                    print("Configured source successfully")
            elif source_type == "crm" and source_sub_type == "salesforce":
                source_obj = SalesforceSource()
                source_obj.set_variables(env_id, storage_id, configuration_file_path, self.secrets_config,
                                         replace_words)
                source_id = source_obj.create_salesforce_source(self)
                if source_id is not None:
                    # Proceed to configure the source connection details
                    if source_obj.configure_salesforce_source_connection(self, source_id, override_configuration_file,
                                                                    read_passwords_from_secrets=read_passwords_from_secrets,
                                                                    env_tag=env_tag,
                                                                    secret_type=secret_type) == "SUCCESS":
                        status = source_obj.test_source_connection(self, source_id)
                        if status == "SUCCESS":
                            status = source_obj.metacrawl_source(self, source_id)
                            if status == "SUCCESS":
                                status = source_obj.configure_tables_and_tablegroups(self, source_id,
                                                                                     override_configuration_file,
                                                                                     export_lookup, self.mappings,
                                                                                     read_passwords_from_secrets,
                                                                                     env_tag=env_tag,
                                                                                     secret_type=secret_type)
                                if status == "SUCCESS":
                                    self.logger.info("Configured source successfully")
                                    print("Configured source successfully")
            else:    #assumes to be cdata source
                source_obj = CdataSource()
                source_obj.set_variables(env_id, storage_id, configuration_file_path, self.secrets_config,
                                         replace_words)
                source_id = source_obj.create_cdata_source(self)
                if source_id is not None:
                    # Proceed to configure the source connection details
                    if source_obj.configure_cdata_source_connection(self, source_id, override_configuration_file,
                                                                    read_passwords_from_secrets=read_passwords_from_secrets,
                                                                    env_tag=env_tag,
                                                                    secret_type=secret_type) == "SUCCESS":
                        status = source_obj.test_source_connection(self, source_id)
                        if status == "SUCCESS":
                            status = source_obj.browse_source_tables(self, source_id)
                            if status == "SUCCESS":
                                status = source_obj.add_tables_to_source(self, source_id)
                                if status == "SUCCESS":
                                    self.logger.info("Added tables to source successfully")
                                    print("Added tables to source successfully")
                                status = source_obj.configure_tables_and_tablegroups(self, source_id,
                                                                                     override_configuration_file,
                                                                                     export_lookup, self.mappings,
                                                                                     read_passwords_from_secrets,
                                                                                     env_tag=env_tag,
                                                                                     secret_type=secret_type)
                                if status == "SUCCESS":
                                    self.logger.info("Configured source successfully")
                                    print("Configured source successfully")

            return response_to_return
        except Exception as e:
            self.logger.error(str(e))
            traceback.print_exc()
            print(str(e))
            return response_to_return

    def __execute(self, thread_number, q):
        while True:
            try:
                print('%s: Looking for the next task ' % thread_number)
                task = q.get()
                print(f'\nThread Number {thread_number} processed {task}')
                entity_type = task["entity_type"]
                if entity_type == "source":
                    replace_words = task["replace_words"] if task["replace_words"] else ""
                    read_passwords_from_secrets = task.get("read_passwords_from_secrets", False)
                    self.cicd_upload_source_configurations(task["source_config_path"],
                                                           override_configuration_file=None,
                                                           export_lookup=True, replace_words=replace_words,
                                                           read_passwords_from_secrets=read_passwords_from_secrets)

            except Exception as e:
                print(str(e))
            q.task_done()

    def cicd_create_sourceartifacts_in_bulk(self, config_base_path, src_replace_words=None,
                                            read_passwords_from_secrets=False):
        """
        Function to configure sources in bulk
        :param config_base_path: Path with all the source configuration dumps
        :param src_replace_words: Pass the strings to be replaced in the configuration file. Example: DEV->PROD;dev->prod
        :param read_passwords_from_secrets: True/False. If True all the source related passwords are read from encrypted file name passed
        """
        num_fetch_threads = 10
        job_queue = queue.Queue(maxsize=100)
        for i in range(num_fetch_threads):
            worker = threading.Thread(target=self.__execute, args=(i, job_queue,))
            worker.setDaemon(True)
            worker.start()

        with open(os.path.join(config_base_path, "modified_files", "source.csv"), "r") as src_files_fp:
            for src_file in src_files_fp.readlines():
                src_args = {"entity_type": "source",
                            "source_config_path": os.path.join(config_base_path, "source", src_file.strip()),
                            "replace_words": src_replace_words,
                            "read_passwords_from_secrets": read_passwords_from_secrets}
                job_queue.put(src_args)
                # print(src_args)
        print('*** Main thread waiting to complete all source configuration requests ***')
        job_queue.join()
        print('*** Done with Source Configurations ***')



from functools import reduce
import json


def deep_get(dictionary, keys, default=None):
    return reduce(lambda d, key: d.get(key, default) if isinstance(d, dict) else default, keys.split("."), dictionary)


def get_data(source_config_path, keys):
    with open(source_config_path, 'r') as f:
        json_dict = json.load(f)
    return deep_get(json_dict, keys)


class CustomError(Exception):
    def __init__(self, message):
        self.message = message
        super(CustomError, self).__init__(self.message)



import os

from infoworks.sdk.cicd.download_configurations.get_source_configuration import DownloadSource
from infoworks.sdk.cicd.download_configurations.lineage import get_lineage_dump
from infoworks.sdk.cicd.download_configurations.utils import Utils
from infoworks.sdk.base_client import BaseClient


class DownloadEntityWithLineage(BaseClient):
    def __init__(self):
        super(DownloadEntityWithLineage, self).__init__()

    def cicd_get_dumps_withlineage(self, workflows_to_dump, pipelines, sources, config_file_dump_path, files_overwrite=True,
                              only_wf_pl=False,
                              maintain_lineage=True,
                              serviceaccountemail="admin@infoworks.io", replace_words=""):
        if not os.path.exists(os.path.join(config_file_dump_path, "modified_files")):
            os.makedirs(os.path.join(config_file_dump_path, "modified_files"))
        if not os.path.exists(os.path.join(config_file_dump_path, "source")):
            os.makedirs(os.path.join(config_file_dump_path, "source"))
        if not os.path.exists(os.path.join(config_file_dump_path, "pipeline")):
            os.makedirs(os.path.join(config_file_dump_path, "pipeline"))
        if not os.path.exists(os.path.join(config_file_dump_path, "workflow")):
            os.makedirs(os.path.join(config_file_dump_path, "workflow"))
        utils_obj = Utils(serviceaccountemail)
        sources_to_dump = []
        pipelines_to_dump = []
        if workflows_to_dump is not None:
            wf_edges = []
            files_dumped = {}
            for workflow_id in workflows_to_dump:
                json_obj = {"entity_id": workflow_id, "entity_type": "workflow"}
                domain_id = utils_obj.get_domain_id(self, json_obj)
                if domain_id:
                    filename, configuration_obj = utils_obj.dump_to_file(self, "workflow", domain_id, workflow_id,
                                                                         replace_words, config_file_dump_path)
                    files_dumped[workflow_id] = filename
                    for task in configuration_obj["configuration"]["workflow"]["workflow_graph"]["tasks"]:
                        if task["task_type"] == "ingest_table_group" and not only_wf_pl:
                            if task["task_properties"]["source_id"] not in sources_to_dump:
                                sources_to_dump.append(task["task_properties"]["source_id"])
                        elif task["task_type"] == "pipeline_build":
                            if task["task_properties"]["pipeline_id"] not in pipelines_to_dump:
                                pipelines_to_dump.append(task["task_properties"]["pipeline_id"])
                        elif task["task_type"] == "build_workflow":
                            if task["task_properties"]["workflow_id"] not in workflows_to_dump:
                                workflows_to_dump.append(task["task_properties"]["workflow_id"])
                                wf_edges.extend([(workflow_id, task["task_properties"]["workflow_id"])])
                        elif task["task_type"] == "export":
                            if task["task_properties"]["export_type"] == "source" and task["task_properties"][
                                "source_id"] not in sources_to_dump and not only_wf_pl:
                                sources_to_dump.append(task["task_properties"]["source_id"])
                            if task["task_properties"]["export_type"] == "pipeline" and task["task_properties"][
                                "pipeline_id"] not in pipelines_to_dump and not only_wf_pl:
                                pipelines_to_dump.append(task["task_properties"]["pipeline_id"])

            if maintain_lineage:
                wf_to_root = []
                for wf in workflows_to_dump:
                    for edge in wf_edges:
                        if wf == edge[-1]:
                            break
                    else:
                        wf_to_root.append(wf)

                workflows_to_dump.append("root")
                for wfid in wf_to_root:
                    wf_edges.append(('root', wfid))
                get_lineage_dump(wf_edges, workflows_to_dump, 'workflow', files_dumped, files_overwrite,
                                 config_file_dump_path)

        if pipelines is not None:
            pipelines_to_dump.extend(pipelines)
            pipelines_to_dump = list(set(pipelines_to_dump))

        if len(pipelines_to_dump) > 0:
            pipeline_edges = []
            files_dumped = {}
            for pipeline_id in pipelines_to_dump:
                json_obj = {"entity_id": pipeline_id, "entity_type": "pipeline"}
                domain_id = utils_obj.get_domain_id(self, json_obj)
                if domain_id:
                    filename, configuration_obj = utils_obj.dump_to_file(self, "pipeline", domain_id, pipeline_id,
                                                                         replace_words, config_file_dump_path)
                    files_dumped[pipeline_id] = filename
                    for node in configuration_obj["configuration"]["pipeline_configs"]["model"].get("nodes", []):
                        req_dict = configuration_obj["configuration"]["pipeline_configs"]["model"]["nodes"][node]
                        if req_dict['type'] == "SOURCE_TABLE":
                            if req_dict['properties']['entity_type'] == "source":
                                src_id = req_dict['properties']['entity_id']
                                if src_id not in sources_to_dump and not only_wf_pl:
                                    sources_to_dump.append(src_id)
                            if req_dict['properties']['entity_type'] == "pipeline_target":
                                pipeline_id_parent = req_dict['properties']['entity_id']
                                if pipeline_id_parent not in pipelines_to_dump and not only_wf_pl:
                                    pipelines_to_dump.append(pipeline_id_parent)
                                pipeline_edges.extend([(pipeline_id_parent, pipeline_id)])

            if maintain_lineage:
                pipeline_to_root = []
                for pipeline in pipelines_to_dump:
                    for edge in pipeline_edges:
                        if pipeline == edge[-1]:
                            break
                    else:
                        pipeline_to_root.append(pipeline)

                pipelines_to_dump.append("root")
                for pid in pipeline_to_root:
                    pipeline_edges.append(('root', pid))
                get_lineage_dump(pipeline_edges, pipelines_to_dump, 'pipeline', files_dumped, files_overwrite,
                                 config_file_dump_path)

        if sources is not None:
            sources_to_dump.extend(sources)
            sources_to_dump = list(set(sources_to_dump))

        if len(sources_to_dump) > 0:
            src_obj = DownloadSource()
            src_obj.cicd_get_sourceconfig_dumps(sources_to_dump, config_file_dump_path, files_overwrite,
                                           serviceaccountemail,
                                           replace_words)



import requests

from infoworks.sdk import url_builder
from infoworks.sdk.url_builder import list_domains_url, list_users_url, add_user_to_domain_url, \
    add_sources_to_domain_url
from infoworks.sdk.utils import IWUtils
from infoworks.core.iw_authentication import get_bearer_token


class Domain:
    def __init__(self, environment_id):
        self.environment_id = environment_id

    def create(self, client, domain_name):
        new_domain_id = ''
        domain_json_object = {
            "name": domain_name,
            "description": "Domain {domain_name} created via API V3".format(domain_name=domain_name),
            "environment_ids": [str(self.environment_id)]
        }
        try:
            create_domain_url = url_builder.create_domain_url(client.client_config)
            response = client.call_api("POST", create_domain_url,
                                       IWUtils.get_default_header_for_v3(client.client_config['bearer_token']),
                                       domain_json_object)
            parsed_response = IWUtils.ejson_deserialize(
                response.content)
            if response.status_code != 200:
                client.logger.info('Cant create domain')
                client.logger.info('Getting the existing domain_id with given name.')
                domains_url_base = list_domains_url(client.client_config)
                filter_condition = str({"name": domain_name})
                domains_url = domains_url_base + f"?filter={{filter_condition}}".format(
                    filter_condition=filter_condition)
                response = client.call_api("GET", domains_url, headers={
                    'Authorization': 'Bearer ' + client.client_config["bearer_token"],
                    'Content-Type': 'application/json'})
                parsed_response = IWUtils.ejson_deserialize(
                    response.content)
                result = parsed_response.get("result", {})
                new_domain_id = None
                if len(result) > 0:
                    new_domain_id = result[0]["id"]
                    existing_env_ids = result[0]["environment_ids"]
                    url_to_update_domain = url_builder.create_domain_url(
                        client.client_config) + f"/{new_domain_id}"
                    config_body = {
                        "name": domain_name,
                        "description": "Domain {domain_name} created via API V3".format(domain_name=domain_name),
                        "environment_ids": existing_env_ids.append(str(self.environment_id))
                    }
                    response = client.call_api("PATCH", url_to_update_domain,
                                               IWUtils.get_default_header_for_v3(client.client_config['bearer_token']),
                                               config_body)
                    if response.status_code == 200:
                        client.logger.info(f"Domain {new_domain_id} updated")
                else:
                    client.logger.error('Can not find domain with given name {} '.format(domain_name))
                client.logger.info('domainId {}'.format(new_domain_id))
            else:
                result = parsed_response.get("result", {})
                new_domain_id = result.get('id')
        except Exception as ex:
            client.logger.error('Response from server: {}'.format(str(ex)))

        return new_domain_id

    def add_user_to_domain(self, client, domain_id, user_id, user_email):
        status_flag = "failed"
        if user_id is None and user_email is not None:
            client.logger.info('Getting userID from given user email....')
            url_for_list_users = list_users_url(client.client_config)
            filter_condition = IWUtils.ejson_serialize({"profile.email": user_email})
            get_userid_url = url_for_list_users + f"?filter={{filter_condition}}".format(
                filter_condition=filter_condition)
            if get_userid_url is not None:
                try:
                    response = requests.request("GET", get_userid_url, headers={
                        'Authorization': 'Bearer ' + client.client_config["bearer_token"],
                        'Content-Type': 'application/json'}, verify=False)
                    if response.status_code == 406:
                        client.client_config['bearer_token'] = get_bearer_token(client.client_config["protocol"],
                                                                                client.client_config["ip"],
                                                                                client.client_config["port"],
                                                                                client.client_config["refresh_token"])
                        headers = IWUtils.get_default_header_for_v3(client.client_config['bearer_token'])
                        response = requests.request("GET", get_userid_url,
                                                    headers=headers, verify=False)
                    client.logger.info(response.json())
                    if response is not None:
                        result = response.json().get("result", [])
                        if len(result) > 0:
                            user_id = result[0]["id"]
                except Exception as e:
                    client.logger.error('Couldnt get result for the user {}'.format(user_email))
                finally:
                    if user_id is None:
                        client.logger.error('Couldnt get result for the user {}'.format(user_email))

        if user_id is not None:
            url_for_adding_user_to_domain = add_user_to_domain_url(client.client_config, user_id)
            client.logger.info('url for adding user to domain - ' + url_for_adding_user_to_domain)
            if url_for_adding_user_to_domain is not None:
                client.logger.info('Adding user {user} to domain {domain}'.format(user=user_id, domain=domain_id))
                try:
                    json_string = IWUtils.ejson_serialize({"entity_ids": [str(domain_id)]})
                    response = requests.post(url_for_adding_user_to_domain, data=json_string,
                                             headers={'Authorization': 'Bearer ' + client.client_config["bearer_token"],
                                                      'Content-Type': 'application/json'},
                                             verify=False)
                    if response.status_code == 406:
                        client.client_config['bearer_token'] = get_bearer_token(client.client_config["protocol"],
                                                                                client.client_config["ip"],
                                                                                client.client_config["port"],
                                                                                client.client_config["refresh_token"])
                        headers = IWUtils.get_default_header_for_v3(client.client_config['bearer_token'])
                        response = requests.post(url_for_adding_user_to_domain, data=json_string,
                                                 headers=headers,
                                                 verify=False)
                    response = response.json()
                    if response.get('message', None) is not None:
                        client.logger.info(response.get('message'))
                        if response.get('message') == "Added Domain(s) to User":
                            status_flag = "success"
                    else:
                        client.logger.error('Error in adding user to domain - {}'.format(response))
                except Exception as e:
                    client.logger.error('Response from server while adding user to domain: {}'.format(user_id))

        return status_flag

    def add_sources_to_domain(self, client, domain_id, source_ids):
        status_flag = "failed"
        url_to_add_source_to_domain = add_sources_to_domain_url(client.client_config, domain_id)
        client.logger.info('url for adding source to domain - ' + url_to_add_source_to_domain)
        if len(source_ids) > 0:
            try:
                source_ids_existing = []
                response = requests.request("GET", url_to_add_source_to_domain,
                                            headers={'Authorization': 'Bearer ' + client.client_config["bearer_token"],
                                                     'Content-Type': 'application/json'}, verify=False)
                if response.status_code == 406:
                    client.client_config['bearer_token'] = get_bearer_token(client.client_config["protocol"],
                                                                            client.client_config["ip"],
                                                                            client.client_config["port"],
                                                                            client.client_config["refresh_token"])
                    headers = IWUtils.get_default_header_for_v3(client.client_config['bearer_token'])
                    response = requests.request("GET", url_to_add_source_to_domain,
                                                headers=headers, verify=False)
                if response is not None:
                    result = response.json().get("result", [])
                    while len(result) > 0:
                        source_ids_existing.extend([item["id"] for item in result])
                        nextUrl = '{protocol}://{ip}:{port}{next}'.format(next=response.json().get('links')['next'],
                                                                          ip=client.client_config['ip'],
                                                                          port=client.client_config['port'],
                                                                          protocol=client.client_config['protocol'],
                                                                          )
                        response = self.callurl(client, nextUrl)
                        result = response.json().get("result", [])
                missing_srcs = set(source_ids) - set(source_ids_existing)
                json_string = IWUtils.ejson_serialize({"entity_ids": list(missing_srcs)})
                response = requests.post(url_to_add_source_to_domain, data=json_string,
                                         headers={'Authorization': 'Bearer ' + client.client_config["bearer_token"],
                                                  'Content-Type': 'application/json'}, verify=False)
                if response.status_code == 406:
                    client.client_config['bearer_token'] = get_bearer_token(client.client_config["protocol"],
                                                                            client.client_config["ip"],
                                                                            client.client_config["port"],
                                                                            client.client_config["refresh_token"])
                    headers = IWUtils.get_default_header_for_v3(client.client_config['bearer_token'])
                    response = requests.post(url_to_add_source_to_domain, data=json_string,
                                             headers=headers, verify=False)
                response = IWUtils.ejson_deserialize(
                    response.content)
                if response.get('message', None) is not None:
                    client.logger.info(response.get('message'))
                    if response.get('message') == "Successfully added accessible sources to the domain":
                        status_flag = "success"
                else:
                    client.logger.error('Error in adding source(s) to domain - {}'.format(response))
            except Exception as e:
                client.logger.error('Response from server while adding source(s) to domain: {}'.format(str(e)))
        return status_flag

    def callurl(self, client, url):
        try:
            print("url {url}".format(url=url))
            response = requests.request("GET", url,
                                        headers={'Authorization': 'Bearer ' + client.client_config["bearer_token"],
                                                 'Content-Type': 'application/json'}, verify=False)
            if response.status_code == 406:
                client.client_config['bearer_token'] = get_bearer_token(client.client_config["protocol"],
                                                                        client.client_config["ip"],
                                                                        client.client_config["port"],
                                                                        client.client_config["refresh_token"])
                headers = IWUtils.get_default_header_for_v3(client.client_config['bearer_token'])
                response = requests.request("GET", url,
                                            headers=headers, verify=False)
            if response is not None:
                return response
        except Exception as e:
            raise Exception("Unable to get response for url: {url}".format(url=url))



import json
import requests
import yaml
from infoworks.sdk.url_builder import get_source_details_url
from infoworks.sdk.utils import IWUtils
from infoworks.sdk.source_response import SourceResponse
from infoworks.sdk.local_configurations import Response

class CSVSource:
    def __init__(self, environment_id, storage_id, source_config_path, secrets=None, replace_words=""):
        self.storage_id = storage_id
        self.environment_id = environment_id
        self.source_config_path = source_config_path
        self.secrets = secrets
        with open(self.source_config_path, 'r') as file:
            json_string = file.read()
            if replace_words != "":
                for key, value in [item.split("->") for item in replace_words.split(";")]:
                    json_string = json_string.replace(key, value)
        self.configuration_obj = IWUtils.ejson_deserialize(json_string)

    def create_csv_source(self, src_client_obj):
        data = self.configuration_obj["configuration"]["source_configs"]
        create_csv_source_payload = {
            "name": data["name"],
            "type": data["type"],
            "sub_type": data["sub_type"],
            "data_lake_path": data["data_lake_path"],
            "data_lake_schema": data["data_lake_schema"] if "data_lake_schema" in data else "",
            "environment_id": self.environment_id,
            "storage_id": self.storage_id,
            "is_source_ingested": True
        }
        if data.get("target_database_name",""):
            create_csv_source_payload["target_database_name"] = data.get("target_database_name","")
        src_create_response = src_client_obj.create_source(source_config=create_csv_source_payload)
        if src_create_response["result"]["status"].upper() == "SUCCESS":
            source_id_created = src_create_response["result"]["source_id"]
            print("Source created successfully")
            return src_create_response
        else:
            src_client_obj.logger.info('Cant create source {} '.format(data["name"]))
            src_client_obj.logger.info(f"Getting the existing SourceId with name {data['name']} if exists")
            filter_condition = IWUtils.ejson_serialize({"name": data['name']})
            source_detail_url = get_source_details_url(
                src_client_obj.client_config) + f"?filter={{filter_condition}}".format(
                filter_condition=filter_condition)
            response = requests.get(source_detail_url,
                                    headers={'Authorization': 'Bearer ' + src_client_obj.client_config['bearer_token'],
                                             'Content-Type': 'application/json'}, verify=False)
            if response.status_code == "406":
                headers = src_client_obj.regenerate_bearer_token_if_needed(
                    {'Authorization': 'Bearer ' + src_client_obj.client_config['bearer_token'],
                     'Content-Type': 'application/json'})
                response = requests.get(source_detail_url, headers=headers, verify=False)
            response = IWUtils.ejson_deserialize(response.content)
            if not response.get('result', None):
                src_client_obj.logger.error("Failed to make an api call to get source details")
                print("Failed to make an api call to get source details")
                src_client_obj.logger.info(response)
                print(response)
                return SourceResponse.parse_result(status=Response.Status.FAILED,
                                                   source_id=None,response=response)
            else:
                src_client_obj.logger.info(
                    f"Source Id with the same Source name {data['name']} : {response['result'][0]['id']}")
                print(f"Source Id with the same Source name {data['name']} : {response['result'][0]['id']}")
                return SourceResponse.parse_result(status=Response.Status.SUCCESS, source_id=response['result'][0]['id'],response=response)

    def configure_csv_source(self, src_client_obj, source_id, mappings, read_passwords_from_secrets=False, env_tag="",
                             secret_type="",config_ini_path=None):
        data = self.configuration_obj["configuration"]["source_configs"]["connection"]
        src_name = str(self.configuration_obj["configuration"]["source_configs"]["name"])
        storage_type = data["storage"]["storage_type"]
        cloud_type = data["storage"].get("cloud_type", None)
        if storage_type == "cloud" and cloud_type == "s3":
            access_id = data["storage"].get("access_id", "")
            secret_key = data["storage"].get("secret_key", "")
            if "source_secrets" in mappings:
                access_id = mappings.get("source_secrets").get("access_id", access_id)
                secret_key = mappings.get("source_secrets").get("secret_key", secret_key)
            if read_passwords_from_secrets and self.secrets["custom_secrets_read"] is True:
                encrypted_key_name = f"{env_tag}-" + src_name
                decrypt_value = self.secrets.get(encrypted_key_name, "")
                if IWUtils.is_json(decrypt_value):
                    decrypt_value_dict = json.loads(decrypt_value)
                    access_id = decrypt_value_dict["access_id"]
                    secret_key = decrypt_value_dict["secret_key"]
            elif read_passwords_from_secrets and self.secrets["custom_secrets_read"] is False:
                encrypted_key_name = f"{env_tag}-" + src_name
                decrypt_value = src_client_obj.get_all_secrets(secret_type, keys=encrypted_key_name,ini_config_file_path=config_ini_path)
                if len(decrypt_value) > 0 and IWUtils.is_json(decrypt_value[0]):
                    decrypt_value_dict = json.loads(decrypt_value[0])
                    access_id = decrypt_value_dict["access_id"]
                    secret_key = decrypt_value_dict["secret_key"]

            source_configure_payload = {
                "source_base_path_relative": data.get("source_base_path_relative",""),
                "source_base_path": data.get("source_base_path",""),
                "storage": {
                    "storage_type": data["storage"]["storage_type"],
                    "cloud_type": data["storage"]["cloud_type"],
                    "access_id": access_id,
                    "secret_key": secret_key,
                    "account_type": data["storage"]["account_type"],
                    "access_type": data["storage"]["access_type"]
                }
            }
        elif storage_type == "cloud" and cloud_type == "wasb":
            source_configure_payload = {}
            pass
        elif storage_type == "cloud" and "project_id" in data["storage"]:
            project_id = data["storage"]["project_id"]
            server_path = data["storage"]["server_path"]
            if "gcp_details" in mappings:
                project_id = mappings["gcp_details"].get("project_id", project_id)
                server_path = mappings["gcp_details"].get("service_json_path", server_path)
            if "gcp_project_id_mappings" in mappings:
                project_id = mappings["gcp_project_id_mappings"].get(data["storage"]["project_id"], project_id)
            if "service_json_mappings" in mappings:
                server_path = mappings["service_json_mappings"].get(data["storage"]["server_path"].split("/")[-1],
                                                                    server_path)

            source_configure_payload = {
                "source_base_path_relative": data.get("source_base_path_relative",""),
                "source_base_path": data.get("source_base_path",""),
                "storage": {
                    "cloud_type": "gs",
                    "storage_type": data["storage"]["storage_type"],
                    "project_id": project_id,
                    "access_type": data["storage"]["access_type"],
                    "server_path": server_path,
                    "upload_option": data["storage"]["upload_option"],
                    "file_details": []
                }
            }
        elif storage_type == "remote":
            # SFTP Source
            data = self.configuration_obj["configuration"]["source_configs"]["connection"]
            source_configure_payload = data
            source_configure_payload["source_base_path"]=""
        else:
            source_configure_payload = {}

        response = src_client_obj.configure_source_connection(source_id, connection_object=source_configure_payload)
        return response

    def import_source_configuration(self, src_client_obj, source_id,
                                    mappings, export_configuration_file=None, export_config_lookup=True,
                                    read_passwords_from_secrets=False):
        src_name = self.configuration_obj["configuration"]["source_configs"]["name"]
        table_group_compute_mappings = mappings.get("table_group_compute_mappings", {})
        source_import_payload = {"configuration": self.configuration_obj["configuration"]}
        modified_table_configs = self.configuration_obj["configuration"]["table_configs"]
        index = 0
        for table_config in self.configuration_obj["configuration"]["table_configs"]:
            modified_table_configs[index]["configuration"]["meta_crawl_performed"] = True
            if not table_config["configuration"].get("meta_crawl_performed", False):
                modified_table_configs[index]["configuration"]["meta_crawl_performed"] = True
                index = index + 1
            else:
                if not table_config["configuration"]["meta_crawl_performed"]:
                    modified_table_configs[index]["configuration"]["meta_crawl_performed"] = True
                index = index + 1
        source_import_payload["configuration"]["table_configs"] = modified_table_configs
        iw_mappings = source_import_payload["configuration"]["iw_mappings"]
        for item in iw_mappings:
            if item.get("entity_type", "") == "environment_compute_template":
                item["recommendation"]["compute_name"] = table_group_compute_mappings.get(
                    item["recommendation"]["compute_name"], item["recommendation"]["compute_name"])
        source_import_payload["configuration"]["iw_mappings"] = iw_mappings
        if export_config_lookup and (export_configuration_file is not None or read_passwords_from_secrets):
            for table in source_import_payload.get("configuration")["table_configs"]:
                # Check if there are any export configurations and passwords to replace
                if table.get("configuration", {}).get("export_configuration", None) is not None:
                    export_configs = table.get("configuration", {}).get("export_configuration")
                    target_type = export_configs.get("target_type", "").upper()
                    table_name = table["configuration"]["name"].upper()
                    override_keys = []
                    try:
                        if export_configuration_file is not None:
                            with open(export_configuration_file) as file:
                                information = yaml.load(file, Loader=yaml.FullLoader)
                            if information["src_export_details"].get(src_name + "_" + table_name, None) is not None:
                                info_key = src_name + "_" + table_name
                                override_keys = information["src_export_details"].get(src_name + "_" + table_name,
                                                                                      {}).keys()
                            else:
                                info_key = src_name
                                override_keys = information["src_export_details"].get(src_name, {}).keys()
                            for key in override_keys:
                                table["configuration"]["export_configuration"]["connection"][key] = \
                                    information["src_export_details"][info_key][key]
                    except Exception as e:
                        src_client_obj.logger.error(
                            f"Failed to lookup the export configuration file {export_configuration_file} due to {str(e)}")

                    try:
                        if target_type.upper() in ["SNOWFLAKE", "POSTGRES"] and read_passwords_from_secrets:
                            # Read the password from KMS
                            encrypted_key_name1 = f"export-configuration-{src_name}-{table['configuration']['name']}"
                            encrypted_key_name2 = f"export-configuration-{src_name}"
                            passwd = self.secrets.get(encrypted_key_name1, "")
                            if passwd == "":
                                passwd = self.secrets.get(encrypted_key_name2, "")
                            table["configuration"]["export_configuration"]["connection"]["password"] = passwd
                        elif target_type.upper() == "BIGQUERY":
                            if "server_path" not in override_keys:
                                server_path = table["configuration"]["export_configuration"].get("connection", {}).get(
                                    "server_path", "")
                                if "gcp_details" in mappings:
                                    server_path = mappings["gcp_details"].get("service_json_path")
                                if "service_json_mappings" in mappings:
                                    server_path = mappings["service_json_mappings"].get(
                                        server_path.split("/")[-1],
                                        server_path)
                                table["configuration"]["export_configuration"]["connection"][
                                    "server_path"] = server_path if server_path != "" else table["configuration"][
                                    "export_configuration"].get("connection", {}).get(
                                    "server_path", "")
                            table["configuration"]["export_configuration"]["connection"][
                                "upload_option"] = "serverLocation"
                    except Exception as e:
                        src_client_obj.logger.error(
                            f"Failed to lookup the export configuration password from secrets due to {str(e)}")
        response = src_client_obj.configure_tables_and_tablegroups(source_id,
                                                                   configuration_obj=source_import_payload.get(
                                                                       "configuration"))
        return response
        # if response["result"]["status"].upper() != "SUCCESS":
        #     src_client_obj.logger.error("Failed to import the source {} (source config path : {})"
        #                                 .format(source_id, self.source_config_path))
        #     src_client_obj.logger.error(response.get("message", "") + "(source config path : {})"
        #                                 .format(self.source_config_path))
        #     return "FAILED"
        # else:
        #     src_client_obj.logger.info(f"Successfully imported source configurations to {source_id}")
        #     return "SUCCESS"



import time

from infoworks.error import PipelineError
from infoworks.sdk import url_builder, local_configurations
from infoworks.sdk.base_client import BaseClient
from infoworks.sdk.local_configurations import Response, ErrorCode
from infoworks.sdk.pipeline_response import PipelineResponse
from infoworks.sdk.utils import IWUtils


class PipelineClient(BaseClient):
    def __init__(self):
        super(PipelineClient, self).__init__()

    def poll_job(self, pipeline_id=None, job_id=None, poll_timeout=local_configurations.POLLING_TIMEOUT,
                 polling_frequency=local_configurations.POLLING_FREQUENCY_IN_SEC,
                 retries=local_configurations.NUM_POLLING_RETRIES):
        failed_count = 0
        response = {}
        timeout = time.time() + poll_timeout
        while True:
            if time.time() > timeout:
                break
            try:
                self.logger.info(f"Failed poll job status count: {failed_count}")
                job_monitor_url = url_builder.get_job_status_url(self.client_config, job_id)
                response = IWUtils.ejson_deserialize(self.call_api("GET", job_monitor_url,
                                                                   IWUtils.get_default_header_for_v3(
                                                                       self.client_config['bearer_token'])).content)
                result = response.get('result', {})
                if len(result) != 0:
                    job_status = result["status"]
                    self.logger.info(
                        "Job poll status : " + result["status"] + "Job completion percentage: " + str(result.get(
                            "percentage", 0)))
                    if job_status in ["completed", "failed", "aborted"]:
                        if job_status == "completed":
                            return PipelineResponse.parse_result(status=Response.Status.SUCCESS,
                                                                 pipeline_id=pipeline_id,
                                                                 job_id=job_id)
                        else:
                            return PipelineResponse.parse_result(status=job_status, pipeline_id=pipeline_id,
                                                                 job_id=job_id)
                else:
                    self.logger.error(f"Error occurred during job {job_id} status poll")
                    if failed_count >= retries - 1:
                        return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                             error_code=ErrorCode.GENERIC_ERROR,
                                                             error_desc=response, job_id=job_id,
                                                             pipeline_id=pipeline_id)
                    failed_count = failed_count + 1
            except Exception as e:
                self.logger.exception("Error occurred during job status poll")
                self.logger.info(str(e))
                if failed_count >= retries - 1:
                    # traceback.print_stack()
                    print(response)
                    raise PipelineError(response.get("message", "Error occurred during job status poll"))
                failed_count = failed_count + 1
            time.sleep(polling_frequency)

    def create_pipeline(self, pipeline_config=None):
        """
        Create a new Pipeline.
        :param pipeline_config: a JSON object containing pipeline configurations
        :type pipeline_config: JSON Object

        pipeline_config_example = {
        "name": "pipeline_name",
        "batch_engine": "spark",
        "domain_id": "domain_id",
        "environment_id":"environment_id. Needed from 5.2 onwards"
        "storage_id": "environment_storage_id",
        "compute_template_id": "environment_compute_id",
        "ml_engine": "SparkML"
        }
        :return: response dict
        """
        response = None
        try:
            if pipeline_config is None:
                self.logger.error('Invalid pipeline configuration. Cannot create a new pipeline.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Cannot create a new pipeline.')
            domain_id = pipeline_config["domain_id"]
            response = IWUtils.ejson_deserialize(self.call_api("POST", url_builder.create_pipeline_url(
                self.client_config, domain_id), IWUtils.get_default_header_for_v3(self.client_config['bearer_token']),
                                                               pipeline_config).content)

            result = response.get('result', {})
            pipeline_id = result.get('id', None)

            if pipeline_id is None:
                self.logger.error('Pipeline failed to create.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Pipeline failed to create.')

            pipeline_id = str(pipeline_id)
            self.logger.info(
                'Pipeline {id} has been created under domain {domain_id}.'.format(id=pipeline_id, domain_id=domain_id))
            return PipelineResponse.parse_result(status=Response.Status.SUCCESS, pipeline_id=pipeline_id,
                                                 response=result)

        except Exception as e:
            self.logger.error('Response from server: ' + str(response))
            self.logger.exception('Error occurred while trying to create a new pipeline.')
            raise PipelineError('Error occurred while trying to create a new pipeline.')

    def get_pipeline(self, pipeline_id, domain_id):
        """
        Gets Infoworks Data pipeline details for given pipeline id
        :param pipeline_id: id of the pipeline whose details is to be fetched
        :type pipeline_id: String
        :param domain_id: Domain id to which the pipeline belongs to
        :type domain_id: String
        :return: response dict
        """
        response = None
        try:
            if pipeline_id is None:
                self.logger.error('Please pass the mandatory pipeline id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory pipeline id as parameter.')
            if domain_id is None:
                self.logger.error('Please pass the mandatory domain id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory domain id as parameter.')
            response = IWUtils.ejson_deserialize(self.call_api("GET", url_builder.get_pipeline_url(
                self.client_config, domain_id, pipeline_id), IWUtils.get_default_header_for_v3(
                self.client_config['bearer_token'])).content)

            result = response.get('result', {})
            # print(response)
            if result.get('id', None) is None:
                self.logger.error('Failed to find the pipeline details')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Failed to find the pipeline details')

            pipeline_id = str(pipeline_id)
            self.logger.info(
                'Successfully got the pipeline {id} details.'.format(id=pipeline_id))
            return PipelineResponse.parse_result(status=Response.Status.SUCCESS, pipeline_id=pipeline_id,
                                                 response=result)

        except Exception as e:
            self.logger.error('Response from server: ' + str(response))
            self.logger.exception('Error occurred while trying to get pipeline details.')
            raise PipelineError('Error occurred while trying to get pipeline details.')

    def delete_pipeline(self, pipeline_id, domain_id):
        """
        Deletes Infoworks Data pipeline  for given pipeline id
        :param pipeline_id: entity id of the pipeline to be deleted
        :type pipeline_id: String
        :param domain_id: Domain id to which the pipeline belongs to
        :type domain_id: String
        :return: response dict
        """
        response = None
        try:
            if pipeline_id is None:
                self.logger.error('Please pass the mandatory pipeline id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory pipeline id as parameter.')
            if domain_id is None:
                self.logger.error('Please pass the mandatory domain id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory domain id as parameter.')
            response = IWUtils.ejson_deserialize(self.call_api("DELETE", url_builder.delete_pipeline_url(
                self.client_config, domain_id, pipeline_id), IWUtils.get_default_header_for_v3(
                self.client_config['bearer_token'])).content)

            result = response.get('result', {})

            if result.get('id', None) is None:
                self.logger.error(f'Failed to delete the pipeline {pipeline_id}')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc=f'Failed to delete the pipeline {pipeline_id}')

            pipeline_id = str(pipeline_id)
            self.logger.info(
                'Successfully deleted the pipeline {id}.'.format(id=pipeline_id))
            return PipelineResponse.parse_result(status=Response.Status.SUCCESS, pipeline_id=pipeline_id)

        except Exception as e:
            self.logger.error('Response from server: ' + str(response))
            self.logger.exception('Error occurred while trying to delete pipeline.')
            raise PipelineError('Error occurred while trying to delete pipeline.')

    def update_pipeline(self, pipeline_id, domain_id, pipeline_config):
        """
        Updates Infoworks Data pipeline details for given pipeline id
        :param pipeline_id: entity id of the pipeline to be updated
        :type pipeline_id: String
        :param domain_id: Domain id to which the pipeline belongs to
        :type domain_id: String
        :param pipeline_config: a JSON object containing pipeline configurations
        :type pipeline_config: JSON Object

        pipeline_config_example = {
        "name": "pipeline_name",
        "compute_template_id": "environment_compute_id",
        "active_version_id": "pipeline_version_id"
        }
        :return: response dict
        """
        response = None
        try:
            if pipeline_config is None:
                self.logger.error('Please pass the mandatory pipeline_config as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory pipeline_config as parameter.')
            if pipeline_id is None:
                self.logger.error('Please pass the mandatory pipeline id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory pipeline id as parameter.')
            if domain_id is None:
                self.logger.error('Please pass the mandatory domain id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory domain id as parameter.')
            response = IWUtils.ejson_deserialize(self.call_api("PUT", url_builder.update_pipeline_url(
                self.client_config, domain_id, pipeline_id), IWUtils.get_default_header_for_v3(
                self.client_config['bearer_token']), pipeline_config).content)

            result = response.get('result', {})

            if result.get('id', None) is None:
                self.logger.error(f'Failed to update the pipeline {pipeline_id}')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc=f'Failed to update the pipeline {pipeline_id}',
                                                     response=response)

            pipeline_id = str(pipeline_id)
            self.logger.info(
                'Successfully updated the pipeline {id}.'.format(id=pipeline_id))
            return PipelineResponse.parse_result(status=Response.Status.SUCCESS, pipeline_id=pipeline_id,
                                                 response=response)

        except Exception as e:
            self.logger.error('Response from server: ' + str(response))
            self.logger.exception('Error occurred while trying to update pipeline.')
            raise PipelineError('Error occurred while trying to update pipeline.')

    def sql_import_into_pipeline(self, pipeline_id, domain_id, pipeline_config):
        """
        Import SQL into given pipeline id
        :param pipeline_id Id of the pipeline to be updated
        :type Id as string
        :param domain_id Domain Id to which the pipeline belongs to
        :type Id as string
        :param pipeline_config: a JSON object containing pipeline configurations
        :type pipeline_config: JSON Object

        pipeline_config_example = {
        "dry_run": "{boolean}",
         "sql": "select * from employee",
         "sql_import_configuration": {
          "quoted_identifier": "DOUBLE_QUOTE",
          "sql_dialect": "LENIENT"
        }
        :return: response dict
        """
        response = None
        try:
            if pipeline_config is None:
                self.logger.error('Please pass the mandatory pipeline_config as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory pipeline_config as parameter.')
            if pipeline_id is None:
                self.logger.error('Please pass the mandatory pipeline id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory pipeline id as parameter.')
            if domain_id is None:
                self.logger.error('Please pass the mandatory domain id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory domain id as parameter.')
            response = IWUtils.ejson_deserialize(self.call_api("POST", url_builder.pipeline_sql_import_url(
                self.client_config, domain_id, pipeline_id), IWUtils.get_default_header_for_v3(
                self.client_config['bearer_token']), pipeline_config).content)

            result = response.get('result', {})

            if result.get('status', 'failed') != 'success':
                self.logger.error(f'Failed to update the pipeline {pipeline_id} with the given SQL')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc=f'Failed to update the pipeline {pipeline_id}',
                                                     response=response)

            self.logger.info(
                'Successfully updated the pipeline {id} with the given SQL.'.format(id=pipeline_id))
            return PipelineResponse.parse_result(status=Response.Status.SUCCESS, pipeline_id=pipeline_id,
                                                 response=response)

        except Exception as e:
            self.logger.error('Response from server: ' + str(response))
            self.logger.exception('Error occurred while trying to update pipeline with the given sql.')
            raise PipelineError('Error occurred while trying to update pipeline with the given sql.')

    def get_pipeline_version_details(self, pipeline_id, domain_id, pipeline_version_id):
        """
        Gets a pipelineVersion with pipeline_version_id of pipeline with {pipeline_id} in domain with {domain_id}
        :param pipeline_id: id of the pipeline whose details is to be fetched
        :type pipeline_id: String
        :param domain_id: Domain id to which the pipeline belongs to
        :type domain_id: String
        :param pipeline_version_id: id of the pipeline_version_id whose details are to be fetched
        :type pipeline_version_id: String
        :return: response dict
        """
        response = None
        try:
            if pipeline_version_id is None:
                self.logger.error('Please pass the mandatory pipeline_version_id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline_version_id configuration. '
                                                                'Please pass the mandatory pipeline_version_id as parameter.')
            if pipeline_id is None:
                self.logger.error('Please pass the mandatory pipeline id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory pipeline id as parameter.')
            if domain_id is None:
                self.logger.error('Please pass the mandatory domain id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory domain id as parameter.')
            response = IWUtils.ejson_deserialize(self.call_api("GET", url_builder.get_pipeline_version_url(
                self.client_config, domain_id, pipeline_id, pipeline_version_id), IWUtils.get_default_header_for_v3(
                self.client_config['bearer_token'])).content)

            result = response.get('result', {})

            if result.get('id', None) is None:
                self.logger.error('Failed to find the pipeline version details')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Failed to find the pipeline version details',
                                                     response=response)

            pipeline_version_id = str(pipeline_version_id)
            self.logger.info(
                'Successfully got the pipeline version {pipeline_version_id} details.'.format(
                    pipeline_version_id=pipeline_version_id))
            return PipelineResponse.parse_result(status=Response.Status.SUCCESS, pipeline_id=pipeline_version_id,
                                                 response=response)

        except Exception as e:
            self.logger.error('Response from server: ' + str(response))
            self.logger.exception('Error occurred while trying to get pipeline version details.')
            raise PipelineError('Error occurred while trying to get pipeline version details.')

    def delete_pipeline_version(self, pipeline_id, domain_id, pipeline_version_id):
        """
        Deletes a pipelineVersion with pipeline_version_id of pipeline with {pipeline_id} in domain with {domain_id}
        :param pipeline_id: id of the pipeline whose details is to be fetched
        :type pipeline_id: String
        :param domain_id: Domain id to which the pipeline belongs to
        :type domain_id: String
        :param pipeline_version_id: id of the pipeline_version_id whose details is to be fetched
        :type pipeline_version_id: String
        :return: response dict

        """
        response = None
        try:
            if pipeline_version_id is None:
                self.logger.error('Please pass the mandatory pipeline_version_id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline_version_id configuration. '
                                                                'Please pass the mandatory pipeline_version_id as parameter.')
            if pipeline_id is None:
                self.logger.error('Please pass the mandatory pipeline id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory pipeline id as parameter.')
            if domain_id is None:
                self.logger.error('Please pass the mandatory domain id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory domain id as parameter.')
            response = IWUtils.ejson_deserialize(self.call_api("GET", url_builder.delete_pipeline_version_url(
                self.client_config, domain_id, pipeline_id, pipeline_version_id), IWUtils.get_default_header_for_v3(
                self.client_config['bearer_token'])).content)

            result = response.get('result', {})

            if result.get('id', None) is None:
                self.logger.error('Failed to delete the pipeline version')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Failed to delete the pipeline version',
                                                     response=response)

            pipeline_version_id = str(pipeline_version_id)
            self.logger.info(
                'Successfully deleted the pipeline version {pipeline_version_id} details.'.format(
                    pipeline_version_id=pipeline_version_id))
            return PipelineResponse.parse_result(status=Response.Status.SUCCESS, pipeline_id=pipeline_version_id,
                                                 response=response)

        except Exception as e:
            self.logger.error('Response from server: ' + str(response))
            self.logger.exception('Error occurred while trying to delete pipeline version details.')
            raise PipelineError('Error occurred while trying to delete pipeline version details.')

    def update_pipeline_version_details(self, pipeline_id, domain_id, pipeline_version_id,
                                        pipeline_version_config):
        """
        Update a pipelineVersion with pipeline_version_id of pipeline with {pipeline_id} in domain with {domain_id}
        :param pipeline_id: id of the pipeline whose details is to be fetched
        :type pipeline_id: String
        :param domain_id: Domain id to which the pipeline belongs to
        :type domain_id: String
        :param pipeline_version_id: id of the pipeline_version_id whose details is to be fetched
        :type pipeline_version_id: String
        :param pipeline_version_config: configurations of the pipeline version to be updated
        :type pipeline_version_config:JSON Object

        pipeline_version_config example:
        {
         "id": "e77850ad5127a2d7dab870ff",
         "created_at": "2019-11-21T12:54:05.875Z",
         "created_by": "6RkfybTRQQByEey3v",
         "modified_at": "2019-11-21T12:54:05.875Z",
         "modified_by": "6RkfybTRQQByEey3v"
        }
        :return: response dict

        """
        response = None
        try:
            if pipeline_version_config is None:
                self.logger.error('Please pass the mandatory pipeline_version_config as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline_version_config configuration. '
                                                                'Please pass the mandatory pipeline_version_config as parameter.')
            if pipeline_version_id is None:
                self.logger.error('Please pass the mandatory pipeline_version_id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline_version_id configuration. '
                                                                'Please pass the mandatory pipeline_version_id as parameter.')
            if pipeline_id is None:
                self.logger.error('Please pass the mandatory pipeline id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory pipeline id as parameter.')
            if domain_id is None:
                self.logger.error('Please pass the mandatory domain id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory domain id as parameter.')
            response = IWUtils.ejson_deserialize(self.call_api("PUT", url_builder.update_pipeline_version_url(
                self.client_config, domain_id, pipeline_id, pipeline_version_id), IWUtils.get_default_header_for_v3(
                self.client_config['bearer_token']), pipeline_version_config).content)

            result = response.get('result', {})

            if result.get('id', None) is None:
                self.logger.error('Failed to update the pipeline version details')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Failed to update the pipeline version details',
                                                     response=response)

            pipeline_version_id = str(pipeline_version_id)
            self.logger.info(
                'Successfully updated the pipeline version {pipeline_version_id} details.'.format(
                    pipeline_version_id=pipeline_version_id))
            return PipelineResponse.parse_result(status=Response.Status.SUCCESS, pipeline_id=pipeline_version_id,
                                                 response=response)

        except Exception as e:
            self.logger.error('Response from server: ' + str(response))
            self.logger.exception('Error occurred while trying to update pipeline version details.')
            raise PipelineError('Error occurred while trying to update pipeline version details.')

    def get_pipeline_configuration_json_export(self, pipeline_id, domain_id):
        """
        Get exported config for pipeline with pipeline_id in domain with domain_id
        :param pipeline_id: id of the pipeline whose details is to be fetched
        :type pipeline_id: String
        :param domain_id: Domain id to which the pipeline belongs to
        :type domain_id: String
        :return: response dict
        """
        response = None
        try:
            if pipeline_id is None:
                self.logger.error('Please pass the mandatory pipeline id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory pipeline id as parameter.')
            if domain_id is None:
                self.logger.error('Please pass the mandatory domain id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory domain id as parameter.')
            response = IWUtils.ejson_deserialize(self.call_api("GET", url_builder.configure_pipeline_url(
                self.client_config, domain_id, pipeline_id), IWUtils.get_default_header_for_v3(
                self.client_config['bearer_token'])).content)

            result = response.get('result', {})

            if result is {}:
                self.logger.error('Failed to get the configuration json of pipeline')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Failed to get the configuration json of pipeline',
                                                     response=response)

            pipeline_id = str(pipeline_id)
            self.logger.info(
                'Successfully got the pipeline {id} configuration json.'.format(id=pipeline_id))
            return PipelineResponse.parse_result(status=Response.Status.SUCCESS, pipeline_id=pipeline_id,
                                                 response=response)

        except Exception as e:
            self.logger.error('Response from server: ' + str(response))
            self.logger.exception('Error occurred while trying to get pipeline configuration json.')
            raise PipelineError('Error occurred while trying to get pipeline configuration json.')

    def update_pipeline_configuration_json_export(self, pipeline_id, domain_id, pipeline_config):
        """
        Import config for pipeline with pipeline_id in domain with domain_id
        :param pipeline_id: id of the pipeline whose details is to be fetched
        :type pipeline_id: String
        :param domain_id: Domain id to which the pipeline belongs to
        :type domain_id: String
        :param pipeline_config: configuration json of the pipeline
        :type pipeline_config: JSON Object
        :return: response dict
        """
        response = None
        try:
            if pipeline_config is None:
                self.logger.error('Please pass the mandatory pipeline_config as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory pipeline_config as parameter.')
            if pipeline_id is None:
                self.logger.error('Please pass the mandatory pipeline id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory pipeline id as parameter.')
            if domain_id is None:
                self.logger.error('Please pass the mandatory domain id as parameter.')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Invalid pipeline configuration. '
                                                                'Please pass the mandatory domain id as parameter.')
            response = IWUtils.ejson_deserialize(self.call_api("POST", url_builder.configure_pipeline_url(
                self.client_config, domain_id, pipeline_id), IWUtils.get_default_header_for_v3(
                self.client_config['bearer_token']), pipeline_config).content)

            result = response.get('result', [])

            if len(result) == 0:
                self.logger.error('Failed to update the configuration json of pipeline')
                return PipelineResponse.parse_result(status=Response.Status.FAILED,
                                                     error_desc='Failed to update the configuration json of pipeline',
                                                     response=response)

            pipeline_id = str(pipeline_id)
            self.logger.info(
                'Successfully updated the pipeline {id} configuration json.'.format(id=pipeline_id))
            return PipelineResponse.parse_result(status=Response.Status.SUCCESS, pipeline_id=pipeline_id,
                                                 response=response)

        except Exception as e:
            self.logger.error('Response from server: ' + str(response))
            self.logger.exception('Error occurred while trying to update pipeline configuration json.')
            raise PipelineError('Error occurred while trying to update pipeline configuration json.')

    def trigger_pipeline_job(self, domain_id, pipeline_id, pipeline_version_id=None, poll=False):
        """
           Trigger the pipeline build job
           :param pipeline_id: entity id of the pipeline
           :type pipeline_id: String
           :param pipeline_version_id: entity id  of the pipeline version
           :type pipeline_version_id: String
           :param domain_id: Domain id to which the pipeline belongs to
           :type domain_id: String
           :param poll: Poll job until its completion
           :type poll: Boolean
        """
        url_for_pipeline_build = url_builder.trigger_pipeline_build_url(self.client_config, domain_id, pipeline_id)
        if pipeline_version_id is not None:
            request_body = {
                "job_type": "pipeline_build",
                "version_id": str(pipeline_version_id)
            }
        else:
            request_body = {
                "job_type": "pipeline_build"
            }
        try:
            response = IWUtils.ejson_deserialize(
                self.call_api("POST", url_for_pipeline_build,
                              IWUtils.get_default_header_for_v3(self.client_config['bearer_token']),
                              request_body).content)
            result = response.get('result', {})
        except Exception as e:
            raise PipelineError(f"Failed to submit pipeline build job for {pipeline_id} " + str(e))
        if len(result) != 0 and "id" in result:
            job_id = result["id"]
            if not poll:
                self.logger.info(f"Pipeline build job has been submitted for {pipeline_id} with {job_id}")
                return PipelineResponse.parse_result(status=Response.Status.SUCCESS)
            else:
                return self.poll_job(pipeline_id=pipeline_id, job_id=job_id)
        else:
            raise PipelineError(f"Failed to submit pipeline build job for {pipeline_id} ")

    def get_pipeline_id(self, pipeline_name, domain_id=None, domain_name=None):
        """
        Function to get pipeline id
        :param pipeline_name: Name of the pipeline
        :param domain_id: Entity identifier of the domain
        :param domain_name: Entity name of the domain
        :return: Pipeline ID
        """
        if domain_id is None and domain_name is None:
            raise Exception(f"Either domain name or domain id has to be passed to get the pipeline id")
        if domain_name is not None and domain_id is None:
            # Find the domain_id
            params = {"filter": {"name": domain_name}}
            url_to_list_domains = url_builder.create_domain_url(
                self.client_config) + IWUtils.get_query_params_string_from_dict(params=params)
            try:
                response = IWUtils.ejson_deserialize(
                    self.call_api("GET", url_to_list_domains,
                                  IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])).content)
                if response is not None:
                    result = response.get("result", [])
                    domain_id = result[0]["id"]
            except Exception as e:
                self.logger.error("Error in listing domains")
                raise PipelineError("Error in listing domains" + str(e))
        try:
            params = {"filter": {"name": pipeline_name}}
            url_to_get_pipeline_info = url_builder.create_pipeline_url(self.client_config,
                                                                       domain_id) + IWUtils.get_query_params_string_from_dict(
                params=params)
            response = IWUtils.ejson_deserialize(
                self.call_api("GET", url_to_get_pipeline_info,
                              IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])
                              ).content)
            if response is not None and "result" in response:
                result = response.get("result", None)
                return result[0].get('id')
        except:
            raise PipelineError("Unable to get pipeline NAME")



from infoworks.error import AdminError
from infoworks.sdk import url_builder
from infoworks.sdk.base_client import BaseClient
from infoworks.sdk.local_configurations import Response, ErrorCode
from infoworks.sdk.generic_response import GenericResponse
from infoworks.sdk.utils import IWUtils


class AdminClient(BaseClient):
    def __init__(self):
        super(AdminClient, self).__init__()
        self._datalineage = {"path": [], "dataflow_objects": [], "master_pipeline_ids": [],
                             "master_sourcetable_ids": []}

    def create_new_user(self, data):
        """
        Function to create new user in Infoworks
        :param data: JSON Payload with user details

        example_data = {
            "profile": {
                "name": "iwx_sdk_user",
                "email": "iwx_sdk_user@infoworks.io",
                "needs_password_reset": False
            },
            "roles": ["modeller"],
            "password": ""
        }

        :return: response dict
        """
        try:
            response = self.call_api("POST",
                                     url_builder.list_users_url(self.client_config),
                                     IWUtils.get_default_header_for_v3(self.client_config['bearer_token']),
                                     data=data)
            parsed_response = IWUtils.ejson_deserialize(
                response.content)
            if response.status_code == 200:
                return GenericResponse.parse_result(status=Response.Status.SUCCESS, response=parsed_response)
            else:
                return GenericResponse.parse_result(status=Response.Status.FAILED,
                                                    error_code=ErrorCode.GENERIC_ERROR,
                                                    error_desc=parsed_response.get("details",
                                                                                   "Error in creating user ")
                                                    )
        except Exception as e:
            self.logger.error("Error in creating user")
            raise AdminError("Error in creating user " + str(e))

    def update_the_user(self, user_id, data):
        """
        Function to update the user
        :param user_id: Entity identifier of the user
        :param data: JSON Payload with user details
        :return: response dict
        """
        try:
            response = self.call_api("PATCH",
                                     url_builder.list_users_url(self.client_config) + f"/{user_id}",
                                     IWUtils.get_default_header_for_v3(self.client_config['bearer_token']),
                                     data=data)
            parsed_response = IWUtils.ejson_deserialize(
                response.content)
            if response.status_code == 200:
                return GenericResponse.parse_result(status=Response.Status.SUCCESS)
            else:
                return GenericResponse.parse_result(status=Response.Status.FAILED,
                                                    error_code=ErrorCode.GENERIC_ERROR,
                                                    error_desc=parsed_response.get("details",
                                                                                   "Error in updating user ")
                                                    )
        except Exception as e:
            self.logger.error("Error in updating user")
            raise AdminError("Error in updating user " + str(e))

    def get_user_details(self, user_id=None, params=None):
        """
        Function to get the user details
        :param user_id: Entity identifier of the user
        :param params: Pass the parameters like limit, filter, offset, sort_by, order_by as a dictionary
        :return: response dict
        """
        if params is None:
            params = {"limit": 20, "offset": 0}
        if user_id is not None:
            url_to_list_user_details = url_builder.list_users_url(self.client_config) + f"/{user_id}"
        else:
            url_to_list_user_details = url_builder.list_users_url(
                self.client_config) + IWUtils.get_query_params_string_from_dict(params=params)
        users_list = []
        try:
            response = IWUtils.ejson_deserialize(
                self.call_api("GET", url_to_list_user_details,
                              IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])).content)
            if response is not None:
                result = response.get("result", [])
                if user_id is not None:
                    users_list.extend([result])
                else:
                    while len(result) > 0:
                        users_list.extend(result)
                        nextUrl = '{protocol}://{ip}:{port}{next}'.format(next=response.get('links')['next'],
                                                                          ip=self.client_config['ip'],
                                                                          port=self.client_config['port'],
                                                                          protocol=self.client_config['protocol'],
                                                                          )
                        response = IWUtils.ejson_deserialize(
                            self.call_api("GET", nextUrl, IWUtils.get_default_header_for_v3(
                                self.client_config['bearer_token'])).content)
                        result = response.get("result", [])
            return GenericResponse.parse_result(status=Response.Status.SUCCESS, response=users_list)
        except Exception as e:
            self.logger.error("Error in listing user information")
            raise AdminError("Error in listing user information" + str(e))

    def add_domains_to_user(self, domain_id, user_id=None, user_email=None):
        """
        Function to make domains accessible to the user
        :param domain_id: Entity identifier for domain
        :param user_id: Entity identifier for user
        :param user_email: email_id of the user
        :return: response dict
        """
        status_flag = "failed"
        if user_id is None and user_email is not None:
            self.logger.info('Getting userID from given user email....')
            url_for_list_users = url_builder.list_users_url(self.client_config)
            filter_condition = IWUtils.ejson_serialize({"profile.email": user_email})
            get_userid_url = url_for_list_users + f"?filter={{filter_condition}}".format(
                filter_condition=filter_condition)
            if get_userid_url is not None:
                try:
                    response = IWUtils.ejson_deserialize(
                        self.call_api("GET", get_userid_url,
                                      IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])).content)
                    self.logger.info(response)
                    if response is not None and "result" in response:
                        result = response.get("result", [])
                        if len(result) > 0:
                            user_id = result[0]["id"]
                except Exception as e:
                    self.logger.error('Couldnt get result for the user {}'.format(user_email))
                    raise AdminError('Couldnt get result for the user {}'.format(user_email))
                finally:
                    if user_id is None:
                        self.logger.error('Couldnt get result for the user {}'.format(user_email))
                        raise AdminError('Couldnt get result for the user {}'.format(user_email))

        if user_id is not None:
            url_for_adding_user_to_domain = url_builder.add_user_to_domain_url(self.client_config, user_id)
            self.logger.info('url for adding user to domain - ' + url_for_adding_user_to_domain)
            if url_for_adding_user_to_domain is not None:
                self.logger.info('Adding user {user} to domain {domain}'.format(user=user_id, domain=domain_id))
                try:
                    json_data = {"entity_ids": [str(domain_id)]}
                    response = IWUtils.ejson_deserialize(
                        self.call_api("POST",
                                      url_for_adding_user_to_domain,
                                      IWUtils.get_default_header_for_v3(self.client_config['bearer_token']),
                                      data=json_data).content)
                    if response.get('message', None) is not None:
                        self.logger.info(response.get('message'))
                        if response.get('message') == "Added Domain(s) to User":
                            status_flag = "success"
                    else:
                        self.logger.error('Error in adding user to domain - {}'.format(response))
                except Exception as e:
                    self.logger.error('Response from server while adding user to domain: {}'.format(user_id))
                    raise AdminError('Response from server while adding user to domain: {}'.format(user_id))
        if status_flag == "success":
            return GenericResponse.parse_result(status=Response.Status.SUCCESS)
        else:
            return GenericResponse.parse_result(status=Response.Status.FAILED)

    def alation_compatible_lineage_for_source(self, src_id, table_id):
        """
        Function to get the lineage of source
        :param src_id: Entity identifier for source
        :param table_id: Entity identifier for table
        :return: source lineage info
        """
        url_to_get_table_details = url_builder.get_table_configuration(self.client_config, src_id, table_id)
        response = self.call_api("GET", url_to_get_table_details,
                                 IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])
                                 )
        parsed_response = IWUtils.ejson_deserialize(
            response.content)
        if response.status_code == 200 and "result" in parsed_response:
            result = parsed_response.get("result", {})
            target_schema_name = result["configuration"]["target_schema_name"]
            target_table_name = result["configuration"]["target_table_name"]
            orig_db_name = result.get("schema_name_at_source", "csv_source_db")
            orig_table_name = result.get("original_table_name", "")

            path_to_add = [[{"otype": "table",
                             "key": f"<DS_ID>.{orig_db_name}.{orig_table_name}"}],
                           [{"otype": "table",
                             "key": f"<DS_ID>.{target_schema_name}.{target_table_name}"}]]
            self._datalineage["path"].append(path_to_add[::])

            for column in result["columns"]:
                orig_name = column["original_name"]
                col_name = column["name"]
                path_to_add = [[{"otype": "column",
                                 "key": f"<DS_ID>.{orig_db_name}.{orig_table_name}.{orig_name}"}],
                               [{"otype": "column",
                                 "key": f"<DS_ID>.{target_schema_name}.{target_table_name}.{col_name}"}]]
                self._datalineage["path"].append(path_to_add[::])
        output = {"dataflow_objects": self._datalineage["dataflow_objects"], "paths": self._datalineage["path"]}
        return output

    def alation_compatible_lineage_for_pipeline(self, domain_id, pipeline_id, pipeline_version_id,
                                                pl_description=""):
        """
        Function to get the lineage of pipeline
        :param domain_id: Entity identifier for domain
        :param pipeline_id: Entity identifier for pipeline
        :param pipeline_version_id: Entity identifier for pipeline version
        :param pl_description: pipeline description
        :return: response dict
        """
        url_to_get_pipelineversion_details = url_builder.list_pipeline_versions_url(self.client_config, domain_id,
                                                                                    pipeline_id) + f"/{pipeline_version_id}"
        response = self.call_api("GET", url_to_get_pipelineversion_details,
                                 IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])
                                 )
        parsed_response = IWUtils.ejson_deserialize(
            response.content)
        if response.status_code == 200 and len(parsed_response.get("result", {})) > 0:
            pipeline_details = parsed_response.get("result", {})
            # hash_object = hashlib.md5(f"{pipeline_id}_{pipeline_version_id}".encode())
            # hashcode = hash_object.hexdigest()
            hashcode = f"{pipeline_id}_{pipeline_version_id}"
            for node in pipeline_details["model"].get("nodes", []):
                req_dict = pipeline_details["model"]["nodes"][node]
                if req_dict['type'].upper() in ["TARGET", "BIGQUERY_TARGET"]:
                    src_node_details = []
                    if req_dict['type'].upper() == "TARGET":
                        target_schema_name = req_dict['properties'].get("target_schema_name", "")
                        target_table_name = req_dict['properties'].get("target_table_name", "")
                    else:
                        target_schema_name = req_dict['properties'].get("dataset_name", "")
                        target_table_name = req_dict['properties'].get("table_name", "")

                    output_columns = [i['name'] for i in req_dict["output_entities"]]
                    for column_name in output_columns:
                        # print(column_name)
                        if not column_name.upper().startswith("ZIW"):
                            url_to_get_pipeline_lineage = url_builder.get_pipeline_lineage_url(self.client_config,
                                                                                               domain_id,
                                                                                               pipeline_id,
                                                                                               pipeline_version_id,
                                                                                               column_name,
                                                                                               node)
                            response = self.call_api("GET", url_to_get_pipeline_lineage,
                                                     IWUtils.get_default_header_for_v3(
                                                         self.client_config['bearer_token'])
                                                     )
                            parsed_response = IWUtils.ejson_deserialize(
                                response.content)
                            if response.status_code == 200 and len(parsed_response.get("result", [])) > 0:
                                graph = parsed_response.get("result", [])
                                temp_output = []
                                source_node = graph[-1]
                                if source_node.get("node_type", "").upper() == "SOURCE_TABLE":
                                    pl_id = source_node.get("pipeline_id", None)
                                    if pl_id is not None and pl_id not in self._datalineage["master_pipeline_ids"]:
                                        self._datalineage["master_pipeline_ids"].append(pl_id)
                                    src_id = source_node.get("source_id", None)
                                    table_id = source_node.get("table_id", "")
                                    if src_id is not None and f"{src_id}:{table_id}" not in self._datalineage[
                                        "master_sourcetable_ids"]:
                                        self._datalineage["master_sourcetable_ids"].append(f"{src_id}:{table_id}")
                                    targetdl_table_name = source_node["target_table_name"]
                                    targetdl_schema_name = source_node["target_schema_name"]
                                    if f"<DS_ID>.{targetdl_schema_name}.{targetdl_table_name}" not in src_node_details:
                                        src_node_details.append(f"<DS_ID>.{targetdl_schema_name}.{targetdl_table_name}")
                                        path_to_add = [[{"otype": "table",
                                                         "key": f"<DS_ID>.{targetdl_schema_name}.{targetdl_table_name}"}],
                                                       [{"otype": "dataflow", "key": f"iwx_pipeline/{hashcode}"}],
                                                       [{"otype": "table",
                                                         "key": f"<DS_ID>.{target_schema_name}.{target_table_name}"}]]
                                        self._datalineage["path"].append(path_to_add[::])
                                    input_port_column = ""
                                    if source_node.get("input_port_column", {}) is not None:
                                        input_port_column = source_node.get("input_port_column", {}).get("name", "")
                                    key = f"<DS_ID>.{targetdl_schema_name}.{targetdl_table_name}.{input_port_column}"
                                    temp = [{"otype": "column", "key": key}]
                                    temp_output.append(temp[::])
                                temp_output.append([{"otype": "dataflow", "key": f"iwx_pipeline/{hashcode}"}])
                                key = f"<DS_ID>.{target_schema_name}.{target_table_name}.{column_name}"
                                temp = [{"otype": "column", "key": key}]
                                temp_output.append(temp[::])
                                self._datalineage["path"].append(temp_output[::])
                        else:
                            # ziw column direct connect to pipeline
                            path_to_add = [[{"otype": "dataflow", "key": f"iwx_pipeline/{hashcode}"}],
                                           [{"otype": "column",
                                             "key": f"<DS_ID>.{target_schema_name}.{target_table_name}.{column_name}"}]]
                            self._datalineage["path"].append(path_to_add[::])
            self._datalineage["dataflow_objects"].append(
                {"external_id": f"iwx_pipeline/{hashcode}", "content": pl_description})
            output = {"dataflow_objects": self._datalineage["dataflow_objects"], "paths": self._datalineage["path"]}
            return output

    def alation_compatible_lineage_for_iwx_metadata(self, domain_id, pipeline_id, sourcetable_ids=None):
        """
        Function to ger complete lineage info
        :param sourcetable_ids: Comma seperated src_id:table_id combination
        :param domain_id: Entity identifier for domain
        :param pipeline_id: Entity identifier for pipeline
        :return: lineage info
        """
        self._datalineage["master_pipeline_ids"].append(pipeline_id)
        # Check if there are any more master pipeline ids
        while len(self._datalineage["master_pipeline_ids"]) > 0:
            parent_pipeline_id = self._datalineage["master_pipeline_ids"].pop()
            response = IWUtils.ejson_deserialize(
                self.call_api("GET", url_builder.get_pipeline_url(self.client_config, domain_id, parent_pipeline_id),
                              IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])
                              ).content)
            if response is not None and "result" in response:
                pipeline_version_id = response.get("result").get("active_version_id")
                pipeline_description = response.get("result").get("description", "")
                self.alation_compatible_lineage_for_pipeline(domain_id, parent_pipeline_id, pipeline_version_id,
                                                             pipeline_description)
        if sourcetable_ids is not None:
            self._datalineage["master_sourcetable_ids"].extend(sourcetable_ids.split(","))
        while len(self._datalineage["master_sourcetable_ids"]) > 0:
            src_table_id = self._datalineage["master_sourcetable_ids"].pop()
            src_id, table_id = src_table_id.split(":")
            self.alation_compatible_lineage_for_source(src_id, table_id)

    def get_environment_details(self, environment_id=None, params=None):
        """
        Function to get environment details
        :param environment_id: Entity identifier of the environment
        :param params: Pass the parameters like limit, filter, offset, sort_by, order_by as a dictionary
        :return: response dict
        """
        if params is None and environment_id is None:
            params = {"limit": 20, "offset": 0}
        url_to_list_environments = url_builder.get_environment_details(
            self.client_config, environment_id) + IWUtils.get_query_params_string_from_dict(params=params)
        env_details = []
        try:
            response = IWUtils.ejson_deserialize(
                self.call_api("GET", url_to_list_environments,
                              IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])).content)
            if response is not None:
                result = response.get("result", [])
                if environment_id is not None:
                    env_details.extend([result])
                else:
                    while len(result) > 0:
                        env_details.extend(result)
                        nextUrl = '{protocol}://{ip}:{port}{next}'.format(next=response.get('links')['next'],
                                                                          ip=self.client_config['ip'],
                                                                          port=self.client_config['port'],
                                                                          protocol=self.client_config['protocol'],
                                                                          )
                        response = IWUtils.ejson_deserialize(
                            self.call_api("GET", nextUrl, IWUtils.get_default_header_for_v3(
                                self.client_config['bearer_token'])).content)
                        result = response.get("result", [])
            return GenericResponse.parse_result(status=Response.Status.SUCCESS, response=env_details)
        except Exception as e:
            self.logger.error("Error in getting environment details")
            raise AdminError("Error in getting environment details" + str(e))

    def get_storage_details(self, environment_id, storage_id=None, params=None):
        """
        Function to get storage details
        :param environment_id: Entity identifier of the environment
        :param storage_id: Entity identifier of the storage
        :param params: Pass the parameters like limit, filter, offset, sort_by, order_by as a dictionary
        :return: response dict
        """
        if params is None and storage_id is None:
            params = {"limit": 20, "offset": 0}
        url_to_list_storages = url_builder.get_environment_storage_details(
            self.client_config, environment_id, storage_id) + IWUtils.get_query_params_string_from_dict(params=params)
        storage_details = []
        try:
            response = IWUtils.ejson_deserialize(
                self.call_api("GET", url_to_list_storages,
                              IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])).content)
            if response is not None:
                result = response.get("result", [])
                if storage_id is not None:
                    storage_details.extend([result])
                else:
                    while len(result) > 0:
                        storage_details.extend(result)
                        nextUrl = '{protocol}://{ip}:{port}{next}'.format(next=response.get('links')['next'],
                                                                          ip=self.client_config['ip'],
                                                                          port=self.client_config['port'],
                                                                          protocol=self.client_config['protocol'],
                                                                          )
                        response = IWUtils.ejson_deserialize(
                            self.call_api("GET", nextUrl, IWUtils.get_default_header_for_v3(
                                self.client_config['bearer_token'])).content)
                        result = response.get("result", [])
            return GenericResponse.parse_result(status=Response.Status.SUCCESS, response=storage_details)
        except Exception as e:
            self.logger.error("Error in getting storage details")
            raise AdminError("Error in getting storage details" + str(e))

    def get_compute_template_details(self, environment_id, compute_id=None, is_interactive=False, params=None):
        """
         Function to get compute template details
         :param environment_id: Entity identifier of the environment
         :param compute_id: Entity identifier of compute cluster
         :param is_interactive: True/False. If True only the details of interactive clusters is fetched
         :param params: Pass the parameters like limit, filter, offset, sort_by, order_by as a dictionary
         :return: response dict
         """
        if params is None and compute_id is None:
            params = {"limit": 20, "offset": 0}
        if is_interactive:
            url_to_list_computes = url_builder.get_environment_interactive_compute_details(
                self.client_config, environment_id, compute_id) + IWUtils.get_query_params_string_from_dict(
                params=params)
        else:
            url_to_list_computes = url_builder.get_environment_compute_details(
                self.client_config, environment_id, compute_id) + IWUtils.get_query_params_string_from_dict(
                params=params)
        compute_details = []
        try:
            response = IWUtils.ejson_deserialize(
                self.call_api("GET", url_to_list_computes,
                              IWUtils.get_default_header_for_v3(self.client_config['bearer_token'])).content)
            if response is not None and "result" in response:
                result = response.get("result", [])
                if compute_id is not None:
                    compute_details.extend([result])
                else:
                    while len(result) > 0:
                        compute_details.extend(result)
                        nextUrl = '{protocol}://{ip}:{port}{next}'.format(next=response.get('links')['next'],
                                                                          ip=self.client_config['ip'],
                                                                          port=self.client_config['port'],
                                                                          protocol=self.client_config['protocol'],
                                                                          )
                        response = IWUtils.ejson_deserialize(
                            self.call_api("GET", nextUrl, IWUtils.get_default_header_for_v3(
                                self.client_config['bearer_token'])).content)
                        result = response.get("result", [])
            return GenericResponse.parse_result(status=Response.Status.SUCCESS, response=compute_details)
        except Exception as e:
            self.logger.error("Error in getting compute template details")
            raise AdminError("Error in getting compute template details" + str(e))

    def get_environment_id_from_name(self, environment_name):
        """
                Function to return environment Id from name
                :param environment_name: Infoworks Environment Name
                :type environment_name: String
                :return: Environment Identifier
        """
        result = \
        self.get_environment_details(environment_id=None, params={"filter": {"name": environment_name}})["result"][
            "response"]
        if len(result) > 0:
            return result[0]["id"]
        else:
            return None

    def get_environment_default_warehouse(self, environment_id):
        """
            Function to return default warehouse name
            :param environment_id: Infoworks Environment Id
            :type environment_id: String
            :return: Environment Name
        """
        result = self.get_environment_details(environment_id=environment_id, params=None)["result"]["response"]
        if len(result) > 0:
            for item in result[0]["data_warehouse_configuration"]["warehouse"]:
                if item["is_default_warehouse"]:
                    return item["name"]
            else:
                return result[0]["data_warehouse_configuration"]["warehouse"][0]["name"]
        else:
            return None

    def get_compute_id_from_name(self, environment_id, compute_name):
        """
                Function to return Environment Compute Id from name
                :param environment_id: Environment identifier
                :type environment_id: String
                :param compute_name: Environment Compute Name
                :type compute_name: String
                :return: Table name
        """
        result = self.get_compute_template_details(environment_id, compute_id=None, is_interactive=True,
                                                   params={"filter": {"name": compute_name}})["result"]["response"]
        if len(result) > 0:
            return result[0]["id"]
        else:
            result = self.get_compute_template_details(environment_id, compute_id=None, is_interactive=False,
                                                       params={"filter": {"name": compute_name}})["result"]["response"]
            if len(result) > 0:
                return result[0]["id"]
        return None

    def get_storage_id_from_name(self, environment_id, storage_name):
        """
                Function to return Environment Storage Id from name
                :param environment_id: Environment identifier
                :type environment_id: String
                :param storage_name: Environment Storage Name
                :type storage_name: String
                :return: Table name
        """
        result = self.get_storage_details(environment_id=environment_id,
                                          params={"filter": {"name": storage_name}})["result"]["response"]
        if len(result) > 0:
            return result[0]["id"]
        else:
            return None

    def create_source_extension(self, data):
        """
            Function to create a source extension
            :param data: Source extension details body
            :type data: String
            :return: response dict
        """
        try:
            response = self.call_api("POST",
                                     url_builder.create_source_extension_url(self.client_config),
                                     IWUtils.get_default_header_for_v3(self.client_config['bearer_token']),
                                     data=data)
            parsed_response = IWUtils.ejson_deserialize(
                response.content)
            if response.status_code == 200:
                return GenericResponse.parse_result(status=Response.Status.SUCCESS)
            else:
                return GenericResponse.parse_result(status=Response.Status.FAILED,
                                                    error_code=ErrorCode.GENERIC_ERROR,
                                                    error_desc=parsed_response.get("details",
                                                                                   f"Error creating source extension {data['name']}")
                                                    )
        except Exception as e:
            self.logger.error(f"Error creating source extension {data['name']}.Please do it by yourself from UI.")
            raise AdminError(f"Error creating source extension  {data['name']}.Please do it by yourself from UI." + str(e))